/*
 *--------------------------------------------------------------------------
 * Copyright Notice:
 *
 * Copyright 1996, 1997, 1998, 1999, 2000 The Regents of the University of 
 * California.  All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the 
 * above copyright notice and the following three paragraphs appear in all 
 * copies.
 *
 * Permission to incorporate this software into commercial products may be
 * obtained from the Office of Intellectual Property Administration, Pete 
 * Ueberoth Building, UCLA, Los Angeles, CA 90095-1594.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGES.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATIONS
 * TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 * 
 *--------------------------------------------------------------------------
 */
/* 
  blockseparation.c
*/

#include "stdio.h"
#include "cif2mat2.h"

extern FILE *gfopen (char *f, char *m);

#define MAX_STRUCT_LISTS 1000

typedef struct {
  int xl,xh,yl,yh,zl,zh;
} Box;

typedef struct {
  Box bbox;
  int good;
} StructInfo;

struct  ListItem { 
  Box b;
  struct ListItem *Next;
};


void processbox( int x1, int x2, int y1, int y2, int z1, int z2); 
void appendboxlist( int i, Box one); 
void printlist(int i); 
void printbox(Box one); 
void removelist(int i);
int  testboxmembership(int list, Box one);
int  intersecttest(Box one, Box two);
Box  determinebbox (Box one, Box two);
void intializeList( void );
int  combinelists( void );
void combine2lists( int i, int j );

struct ListItem *StructList[MAX_STRUCT_LISTS];
StructInfo sl[MAX_STRUCT_LISTS];
int listcnt;
int firsttime;
FILE *fp;

/**********************************************************/
/**********************************************************/

void blockseparation()
{
  char buf[100];
  int  xh,xl,yh,yl,zh,zl;
  int  status;
  int i;
  FILE *p;

  p = gfopen ( PREMASONFILE,"r");

  //debug = 0;
  firsttime = 1;
  (void) intializeList();

  while (fgets(buf, 100, p) != NULL) {
    sscanf (buf, "%d%d%d%d%d%d", &xl,&xh,&yl,&yh,&zl,&zh);
    processbox( xl,xh,yl,yh,zl,zh);
  }
  fclose (p);

  while ( combinelists() ){ }

  fp = gfopen ( POSTMASONFILE,"w");

  for (i= 0; i< listcnt; i++) {
    if (sl[i].good){
      //printf("bb: "); printbox (sl[i].bbox);
      printlist( i );   // Now print out the list
    }
    //removelist( i );  // Release Dynamic Memory of the list.
  }

  fclose (fp);

}

/**********************************************************/

int combinelists()
{
  struct ListItem *p;
  int i, j;

  for (i= 0; i< listcnt; i++) {
    if (sl[i].good) {
      for ( p= StructList[i] ; p != 0 ; p = p ->Next ) {  
        for (j= 0; j< listcnt; j++) {
          if (i != j && sl[j].good) {
            //printf ("testing %d %d.\n", i, j);
            //if (i==3 && j==5) {debug =1;}
            //else              {debug =0;}
            if (testboxmembership (j,p->b) ){
              combine2lists(i,j);
              //printf ("-->combined %d %d<--\n", i, j);
              return 1;
            } 
          }
        }
      }
    }
  }
  return 0;
}

/**********************************************************/
void combine2lists( int i, int j)
{
  struct ListItem *item, *last;

  /* find the last item of the 1st list */
  for( last = StructList[i] ; last->Next != 0 ; last = last->Next ) 
  { }

  item = StructList[j];
  last->Next = item;

  //removelist (j);
  sl[j].good = 0;
}

/**********************************************************/
void intializeList()
{
  int i;
  for (i=0; i<MAX_STRUCT_LISTS; i++)
    StructList[i] = 0;
}

/**********************************************************/
int testboxmembership (int i, Box b)
{
  struct ListItem *p;

#ifdef CIF2MAT_DEBUG
    printf ("------------------------------------\n");
    printf ("in testboxmembership\n");
    printbox (sl[i].bbox);
    printbox (b);
    printf ("------------------------------------\n");
#endif

  if (intersecttest (sl[i].bbox, b) ){
    //printf ("box and bounding box intersects\n");
    for ( p= StructList[i] ; p != 0 ; p = p ->Next ) {  
      if (intersecttest ( b, p->b )) 
        return 1;
    }
  }
  return 0;
}

/**********************************************************/
void processbox(int xl, int xh, int yl, int yh, int zl, int zh)
{
  int i;
  Box b;

  b.xl = xl; b.xh = xh;
  b.yl = yl; b.yh = yh;
  b.zl = zl; b.zh = zh;

  if (firsttime) {
    firsttime = 0;
    listcnt = 0;
  }
  else {
    for (i=0; i<listcnt; i++) { 
      if (testboxmembership (i, b) ){
        appendboxlist (i, b);
        return; 
      }
    }
  }

  /* not part of any existing lists, create a new list */
  appendboxlist (listcnt, b);
  listcnt++;
}

/**********************************************************/
void appendboxlist( int i, Box b)
{
  struct ListItem *item, *last;
  Box newbb;

  item = (struct ListItem *) malloc(sizeof(struct ListItem));
  if ( item==0 )  {  printf ("Malloc Failed \n"); return ; }

  /* Copy box information into new item */
  item->b = b;

  /* Set that next Item is nothing */
  item->Next = 0; 

  if ( StructList[i] == 0 ) {  
    StructList[i] = item;
    sl[i].bbox = b;
    sl[i].good = 1;
    return;
  }

  /* determine the new bounding box of the boxes in the list */
  sl[i].bbox = determinebbox ( sl[i].bbox, item->b ); 

  /* find the last item of the list */
  for( last = StructList[i] ; last->Next != 0 ; last = last->Next ) 
  { }

  /* append new item to last item  */
  last ->Next = item;


  return;
}

/**********************************************************/
void printbox (Box b)
{
  fprintf(fp, "\t%d\t%d\t%d\t%d\t%d\t%d\n", 
         b.xl,b.xh,b.yl,b.yh,b.zl,b.zh );
}

/**********************************************************/
void printlist( int i)  
{   
  struct ListItem *p;

  for ( p= StructList[i] ; p != 0 ; p = p ->Next ) {  
    printbox ( p->b );
    //printf("List %d ->\t%d\t%d\t%d\t%d\t%d\t%d\n", 
    //        i, p->b.xl,p->b.xh,p->b.yl,p->b.yh,p->b.zl,p->b.zh );
  }
  fprintf (fp, "\n");

}

/**********************************************************/

void removelist(int i)
{   
  struct ListItem *p,*next;

  if ( StructList[i]== 0 || !sl[i].good) {  return;  } // If no List return

  for(p = StructList[i] ; p !=0 ; p = next ) {
    next = p->Next; // Save the next item position
    free ( p );     // release the item at p.
  }

  StructList[i]=0;  // Make sure that list point to 0 (Empty list)

}

/**********************************************************/
Box determinebbox (Box bb, Box i)
{
  Box newbb;
  
  newbb.xl = MIN(i.xl, bb.xl);
  newbb.xh = MAX(i.xh, bb.xh);
  newbb.yl = MIN(i.yl, bb.yl);
  newbb.yh = MAX(i.yh, bb.yh);
  newbb.zl = MIN(i.zl, bb.zl);
  newbb.zh = MAX(i.zh, bb.zh);

  return newbb; 
}

/**********************************************************/

int intersecttest ( Box a, Box b )
{
  int xint, yint, zint;

  xint = 0;
  yint = 0;
  zint = 0;

  if (((a.xh >= b.xh)&&(a.xl <= b.xh)) || ((a.xh >= b.xh)&&(a.xl <= b.xl))) 
     xint = 1;
  if (((b.xh >= a.xh)&&(b.xl <= a.xh)) || ((b.xh >= a.xh)&&(b.xl <= a.xl))) 
     xint = 1;

  if (((a.yh >= b.yh)&&(a.yl <= b.yh)) || ((a.yh >= b.yh)&&(a.yl <= b.yl))) 
     yint = 1;
  if (((b.yh >= a.yh)&&(b.yl <= a.yh)) || ((b.yh >= a.yh)&&(b.yl <= a.yl))) 
     yint = 1;

  if (((a.zh >= b.zh)&&(a.zl <= b.zh)) || ((a.zh >= b.zh)&&(a.zl <= b.zl))) 
     zint = 1;
  if (((b.zh >= a.zh)&&(b.zl <= a.zh)) || ((b.zh >= a.zh)&&(b.zl <= a.zl))) 
     zint = 1;

  if (xint && yint && zint)
     return 1;
  else
     return 0;
}
