/*
 *--------------------------------------------------------------------------
 * Copyright Notice:
 *
 * Copyright 1996, 1997, 1998, 1999, 2000 The Regents of the University of 
 * California.  All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the 
 * above copyright notice and the following three paragraphs appear in all 
 * copies.
 *
 * Permission to incorporate this software into commercial products may be
 * obtained from the Office of Intellectual Property Administration, Pete 
 * Ueberoth Building, UCLA, Los Angeles, CA 90095-1594.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGES.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATIONS
 * TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 * 
 *--------------------------------------------------------------------------
 */
/* 
  processcif.c
*/

#include "stdio.h"
#include "string.h"
#include "math.h"

#include "cif2mat2.h"

float EPS_Scale;
char temp[500];
double EPSgrid;
float grid,minx,miny;
int cellindex;
FILE *lfile;

extern FILE *openlayerfile (char *f, char *m);

/**********************************************************/
int getfilechar(FILE *fp)
{
  int x;
  x=fgetc(fp);
  if (x==':') x='_';
  return(x);
}

/**********************************************************/
void Err(char *s)
{
  printf("Error: %s \n",s);
}

/**********************************************************/
int Number(int input)
{
  if (strchr("1234567890-+Ee.",input)==NULL) return(FALSE);
  else return(TRUE);
}

/**********************************************************/
int Space(int input)
{
  //Tab,cr,lf,space  ASCII
  if (isalpha(input) || strchr("_()1234567890-+Ee.",input)!=NULL) 
    return(FALSE);
  return(TRUE);
}

/**********************************************************/
int NextNumber(FILE *fpp,int maxsize,char *s,float *f)
{
int i;
char x;
//char ss[100];

x=getfilechar(fpp); 
//sprintf(ss,"%p ",fpp);
//Stat(ss);
//sprintf(ss,"%c ",x);
//Stat(ss);

while(Space(x)==TRUE)
	{
	x=getfilechar(fpp);
   //sprintf(ss,"s:%c ",x);
   //Stat(ss);
	if (feof(fpp))
   	{
      //Err("Unexpected end of file");
      return(FALSE);
      }
	}
i=0;
s[i]=x;
while(Space(x)==FALSE)
	{
	i++;
	x=getfilechar(fpp);
   //sprintf(ss,"r:%c ",x);
   //Stat(ss);
   if (i>=maxsize) break;
	if (feof(fpp)) return(FALSE);
	s[i]=x;
	}
s[i]=0;
(*f)=0;
if (Number(s[0])==TRUE) (*f)=atof(s);
return(TRUE);
}

/**********************************************************/
void EPSWire(float w)
{
int i,xx,yy;
float tempx,tempy;
float roundx,roundy;

if (points==0) return;
//EPS_Scale=1.0;

//START LINE
fprintf(out,"newpath\n");
//LINE DATA
for(i=0;i<points;i++)
	{
	xx=EPS_Scale*(point[i].x-minx);
	yy=EPS_Scale*(point[i].y-miny);
	roundx=(xx);// /25.4*72*EPSgrid;
	roundy=(yy);// /25.4*72*EPSgrid;
	if (i==0) fprintf(out,"%i %i moveto\n",
		(long int)roundx,(long int)roundy);
	else fprintf(out,"%i %i lineto\n",
		(long int)roundx,(long int)roundy);
	}
//END LINE
fprintf(out,"%i setlinewidth\n",(long int)(w*EPS_Scale));
fprintf(out,"stroke\n");
fprintf(out,"%i setlinewidth\n",1);
}

/**********************************************************/
void EPSLine()
{
  int i,xx,yy;
  float tempx,tempy;
  float roundx,roundy;

  if (points==0) return;
  //EPS_Scale=1.0;

  //START LINE
  fprintf(out,"newpath\n");
  //LINE DATA
  for(i=0;i<points;i++)
	{
	xx=EPS_Scale*(point[i].x-minx);
	yy=EPS_Scale*(point[i].y-miny);
	roundx=(xx);// /25.4*72*EPSgrid;
	roundy=(yy);// /25.4*72*EPSgrid;
	if (i==0) fprintf(out,"%i %i moveto\n",
		(long int)roundx,(long int)roundy);
	else fprintf(out,"%i %i lineto\n",
		(long int)roundx,(long int)roundy);
	}
  //END LINE
  fprintf(out,"closepath fill \n");
}

/**********************************************************/
void EPSWriteCell(char *name,double NewGrid)
{
  fprintf(out,"/%s \n {",name);
  //if (EPSgrid==NewGrid) return;
  if (EPSgrid!=0)
	{
	fprintf(out,"%.9f %.9f scale \n",1.0/EPSgrid,1.0/EPSgrid);
	//printf("%.9f %.9f scale \n",1.0/EPSgrid,1.0/EPSgrid);
	}
  EPSgrid=NewGrid;
  fprintf(out,"%.9f %.9f scale \n",EPSgrid,EPSgrid);
  //printf("%.9f %.9f scale \n",EPSgrid,EPSgrid);
}

/**********************************************************/
void EPSCallCell(int cellnumb,long int translatex,long int translatey)
{
  int i;

  for(i=0;i<cellindex;i++)
	if (cell[i].number==cellnumb) break;
  if (i==cellindex) 
	{
	printf("Cannot find cell # : %i ",cellnumb);
	return;
	}
  if (translatex!=0 || translatey!=0) 
	{
	fprintf(out,"%i %i translate\n ",
		(long int)(translatex*EPS_Scale),
		(long int)(translatey*EPS_Scale));
	//printf("%i %i translate\n ",translatex,translatey);
	}
  fprintf(out,"%s \n ",cell[i].name);
  //printf("%s \n ",cell[i].name);
  if (translatex!=0 || translatey!=0) 
	{
	fprintf(out,"%i %i translate\n ",
		(long int)(-1*translatex*EPS_Scale),
		(long int)(-1*translatey*EPS_Scale));
	//printf("%i %i translate\n ",-1*translatex,-1*translatey);
	}
}

/**********************************************************/
void CIFReadLine()
{
  float xx,yy;
  int i,errval;

  i=0;
  for(;;)
	{
	errval=NextNumber(in,100,temp,&xx);
	//printf("ReadLine: %s \n",temp);
	if (Number(temp[0])==FALSE) break;
	if (errval==FALSE) break;
	errval=NextNumber(in,100,temp,&yy);
	if (errval==FALSE) break;
	//Convert into mm
	point[i].x=xx;//*grid;
	point[i].y=yy;//*grid;
	if (i<MAXPTS) i++;
	else
		{
		printf("Too many points in line.\r\n");
		break;
		}
	}
  //backup the file pointer
  fseek(in,(long)(-1.0*strlen(temp)-2),SEEK_CUR);
  //printf("bakcing up %i ",(int)(-1.0*strlen(temp)-2));
  //getch();
  points=i;
}

/**********************************************************/
/*
void CIFReadWire()
{
  float DX,DY,x1,x2,y2,y1,theta;
  int i,errval;

  i=0;
  errval=NextNumber(in,100,temp,&w);
  //printf("ReadLine: %s \n",temp);
  if (Number(temp[0])==FALSE) return;
  if (errval==FALSE) return;

  //Read in the first coordinate
  errval=NextNumber(in,100,temp,&x1);
  if (errval==FALSE) return;
  errval=NextNumber(in,100,temp,&y1);
  if (errval==FALSE) return;

  printf("Width: %f x:%f y:%f \n",w,x1,y1);

  for(;;)
	{
	errval=NextNumber(in,100,temp,&x2);
	if (Number(temp[0])==FALSE) break;
	if (errval==FALSE) break;
	errval=NextNumber(in,100,temp,&y2);
	if (errval==FALSE) break;

	theta=atan2((x1-x2),(y1-y2));
	DX=w/2.0*cos(theta);
	DY=w/2.0*sin(theta);
	printf("theta: %f DX: %f DY:%f x2:%f y2:%f\n",theta,DX,DY,x2,y2);
	i=0;
	point[i].x=x2+DX; point[i].y=y2+DY;i++;
	point[i].x=x2-DX; point[i].y=y2-DY;i++;
	point[i].x=x1-DX; point[i].y=y1+DY;i++;
	point[i].x=x1+DX; point[i].y=y1+DY;i++;
	point[i].x=x2+DX; point[i].y=y2+DY;i++;
	points=i;
	EPSLine();
	x1=x2;
	y1=y2;
	}
  //backup the file pointer
  fseek(in,(long)(-1.0*strlen(temp)-2),SEEK_CUR);
  //printf("bakcing up %i ",(int)(-1.0*strlen(temp)-2));
  //getch();
  points=i;
}
*/

/**********************************************************/
void CIFReadWire(float *w)
{
  float DX,DY,x1,x2,y2,y1,theta;
  int i,errval;

  i=0;
  errval=NextNumber(in,100,temp,w);
  //printf("ReadLine: %s \n",temp);
  if (Number(temp[0])==FALSE) return;
  if (errval==FALSE) return;

  for(;;)
	{
	errval=NextNumber(in,100,temp,&x2);
	if (Number(temp[0])==FALSE) break;
	if (errval==FALSE) break;
	errval=NextNumber(in,100,temp,&y2);
	if (errval==FALSE) break;
	point[i].x=x2; 
	point[i].y=y2;
	i++;
	}
  //backup the file pointer
  fseek(in,(long)(-1.0*strlen(temp)-2),SEEK_CUR);
  //printf("bakcing up %i ",(int)(-1.0*strlen(temp)-2));
  //getch();
  points=i;
}

/**********************************************************/
void CIFReadBox()
{
  char token, mytmpstr[80];
  int ccnt;
  int i, num[4];
  float xl, xh, yl, yh;

  if ( fgetc(in) ==  ' ' ){
    //fscanf(in, "%d %d %d,%d;", &a,&b,&c,&d);
    //fprintf(out, "    B %d %d %d %d;\n", a, b , c, d);

    for (i=0; i<4; i++){
      ccnt = 1;
      token = fgetc(in); 
      while ( isdigit(token) || token == '-' ) {
        ccnt++;
        token = fgetc(in);
      }
      fseek (in, -ccnt, SEEK_CUR);
      fgets (mytmpstr, ccnt, in);
      sscanf(mytmpstr, "%d", &num[i]);
      fgetc(in);
    } 
    //fprintf(OUTPUTTO,"%d %d %d %d\n", num[0], num[1], num[2], num[3]);
#define NUMVERTICES 4
#define HOLEFLAG    0
    fprintf(lfile, "%d\n",NUMVERTICES, HOLEFLAG);
    xh = (float)(num[2]+num[0]/2.0)/100.;
    xl = (float)(num[2]-num[0]/2.0)/100.;
    yh = (float)(num[3]+num[1]/2.0)/100.;
    yl = (float)(num[3]-num[1]/2.0)/100.;
    fprintf(lfile, "%.1f %.1f\n", xl, yl);
    fprintf(lfile, "%.1f %.1f\n", xh, yl);
    fprintf(lfile, "%.1f %.1f\n", xh, yh);
    fprintf(lfile, "%.1f %.1f\n", xl, yh);
  }
}

/**********************************************************/
void CIFReadLayer()
{
  char layer[8], myfilename[12];
  char token;
  int ccnt, bcnt;

  if ( fgetc(in) ==  ' ' ){
    bcnt = 0;
    ccnt = 1;
    while ( fgetc(in) != ';'){ ccnt++; }
    fseek (in, -ccnt, SEEK_CUR);
    fgets (layer, ccnt, in);
    //fprintf(OUTPUTTO, "L %s", layer);

    /* search forward to see how many boxes are in this layer */
    ccnt = 1;
    token = fgetc(in); ccnt++; 
    while ( token != EOF ){ 
      if (token == ';') {  
        token = fgetc(in); ccnt++; 
        while (!isalpha(token)) { token = fgetc(in); ccnt++; }
        if (token == 'B'){
          bcnt++; 
        }
        else {
          break;
        }
      }
      token = fgetc(in); ccnt++; 
    }
    //fprintf(OUTPUTTO, "L %s --> %d\n", layer, bcnt);
    fseek (in, -ccnt, SEEK_CUR);

    if ( lfile ) { fclose (lfile); }

    if (bcnt) {
      strcpy (myfilename, layer);
      strcat (myfilename, ".gpc");
      lfile = openlayerfile ( myfilename, "w");
      //fprintf (lfile, "%s\n", layer); 
      //fprintf (lfile, "box count %d\n", bcnt); 
      fprintf (lfile, "%d\n", bcnt); 
    }
  }

  return;
}

/**********************************************************/
char CIFReadCell(double *NewGrid)
{
  float tempval;
  int errval,a,b;

  //Format: 
  //DS index a b "where a and b are scaling coefs 
  //9 name

  errval=NextNumber(in,100,temp,&tempval);
  if (errval==FALSE) return(FALSE);
  cell[cellindex].number=(int)tempval;

  //Read in a and b
  errval=NextNumber(in,100,temp,&tempval);
  if (errval==FALSE) 
	{
	Err ("error reading a");
	return(FALSE);
	}
  a=(int)tempval;
  errval=NextNumber(in,100,temp,&tempval);
  if (errval==FALSE) 
	{
	Err ("error reading b");
	return(FALSE);
	}
  b=(int)tempval;

  //printf("Found a scale with the grid: %f,%f = %E (mm) \n",
  //	(float)a,(float)b,(float)(1E-5/b*a));
  (*NewGrid)=(float)(1E-5/b*a)*72/25.4;

  //Read in the flag '9'
  errval=NextNumber(in,100,temp,&tempval);
  if (temp[0]!='9') 
	{
	//printf("ReadCell: %s",temp);
	Err("Critical error.  Name flag not found after DS statement");
	return (FALSE);
	}
  errval=NextNumber(in,100,temp,&tempval);
  strcpy(cell[cellindex].name,temp);
  //printf("Read the name: %s",temp);
  //fseek(in,(long)(-1.0*strlen(temp)-2),SEEK_CUR);
  return(TRUE);
}

/**********************************************************/
char CIFCallCell(int *index,long int *x, long int *y)
{
  float tempval;
  int errval;

  //Format: 
  //C index T x y 
  // where index is the called cell and x and y are the translations.

  errval=NextNumber(in,100,temp,&tempval);
  if (errval==FALSE) return(FALSE);
  (*index)=(int)tempval;
  //printf("Read in the cell %i \n",(*index));
  //Read in the flag 'T'
  errval=NextNumber(in,100,temp,&tempval);
  if (temp[0]!='T') 
	{
	//printf("No translation.");
	(*x)=(long int)0;
	(*y)=(long int)0;
	fseek(in,(long)(-1.0*strlen(temp)-2),SEEK_CUR);
	return(TRUE);
	}
  //Read in x and y
  errval=NextNumber(in,100,temp,&tempval);
  if (errval==FALSE) return(FALSE);
  (*x)=(long int)tempval;
  errval=NextNumber(in,100,temp,&tempval);
  if (errval==FALSE) return(FALSE);
  (*y)=(long int)tempval;
  //printf("translation: %i %i \n",(*x),(*y));
  //fseek(in,(long)(-1.0*strlen(temp)-2),SEEK_CUR);
  return(TRUE);
}

/**********************************************************/

void processcif()
{
  char token;
  char buf[100];
  float f,w;
  long int translatex,translatey;
  int callindex;
  double NewGrid;
  int errval,xx,i;

  while ( (token = fgetc(in)) != EOF ){ 
      switch ( token ) {

        case '9':  
          fgets(buf, 100, in); 
          break;

        case 'B':
  	  CIFReadBox();
          break;

        case 'C':
          //if (temp[0]=='C' && temp[1]==0) {
          ////for(i=0;i<strlen(temp);i++) printf("%c=%i ",temp[i],temp[i]);
          //  if (CIFCallCell(&callindex,&translatex,&translatey)==TRUE) {
          //    EPSCallCell(callindex,translatex,translatey);
          //  }
          //}
          break;

        case 'D':
          //  if (strstr(temp,"DF")!=NULL) {
          //     fprintf(out,"} def\n\n");
          //  }
          //  else if (strstr(temp,"DS")!=NULL) {
          //    if (CIFReadCell(&NewGrid)==TRUE); {
          //       EPSWriteCell(cell[cellindex].name,NewGrid);
          //       if (cellindex<100) 
          //         cellindex++;
	  //       else {
          //         Err("Critical Error.  Too many cells.");
          //         return;
          //      }
          //    }
          //  }
          break;

        case 'E':
          /* ignoring these (and other) cif commands by */
          /* read in and discard the reminder of that line */
          fgets(buf, 100, in); 
          fprintf(OUTPUTTO, "%s", buf);
          if ( lfile ) { fclose (lfile); }
          break;

        case 'L':
  	  CIFReadLayer();
          break;

        case 'P':
          CIFReadLine();
          break;

        case 'W':
          CIFReadWire(&w);
          break;

        case '(':
          fgets(buf, 100, in); 
          break;

        default:
          break;
      }
  }

  return;
}
