/*
 *--------------------------------------------------------------------------
 * Copyright Notice:
 *
 * Copyright 1996, 1997, 1998, 1999, 2000 The Regents of the University of 
 * California.  All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the 
 * above copyright notice and the following three paragraphs appear in all 
 * copies.
 *
 * Permission to incorporate this software into commercial products may be
 * obtained from the Office of Intellectual Property Administration, Pete 
 * Ueberoth Building, UCLA, Los Angeles, CA 90095-1594.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGES.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATIONS
 * TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 * 
 *--------------------------------------------------------------------------
 */

/* processstructures.c for cif2mat */

#include <stdio.h>
#include <math.h>

#include "cif2mat2.h"

#define MAXSTRUCTHINGES 20

#define XPIVOT       8
#define YPIVOT       4
#define ZPIVOT       2

typedef struct pivotaxis_ty {
  short pivot; /* axis of pivot, x=8, y=4, z=2 */ 
  float x;
  float y;
  float z;
} pivotaxis_ty;

typedef struct geom_info_ty {
  int min_a;
  int max_b;
  int min_c;
  int max_d;
} geom_info_ty;

typedef struct coord {
  short matchpsuedo;   /*  match/psuedo layer (top=2) (bottom=1) */
  int a;               /* xmin */
  int b;               /* xmax */
  int c;               /* ymin */
  int d;               /* ymax */
} coord;

typedef struct boxty {
  short on;            /* match/psuedo layer (top=2) (bottom=1) */
  int a;               /* xmin */
  int b;               /* xmax */
  int c;               /* ymin */
  int d;               /* ymax */
  int e;               /* zmin */
  int f;               /* zmax */
} boxty;

typedef struct structinfoty {
  short anchored;            /* is this structure/object anchored */
  short hinged;              /* is this structure/object hinged */
  geom_info_ty geom;         /* contains the bbox/geomtry info for struct */
  int   linkedto;            /* if hinged, which struct is it linked to */ 
  int   linkage[10];         /* chain of links */
  int   linkagecnt;          /* how many links in the chain */
  int   hingecnt;            /* how many hinges are in this structure */
  coord hcoord[MAXSTRUCTHINGES];/* position info of each hinge in structure */
  boxty reflector;           /* reflector information in each structure */
  int   ncnt;                /* how many neighboring structures */
  int   narray[10];          /* who are my neighbors */
  int   lcnt;                /* how many "boxes" made up this structure */
  short done;                /* done posting/displaying this structure */
} structinfoty;

char   braces[30];
char   tmpstr[100];
short  geom1st;              /* 1st time flag through each struct */
int    indentlevel;          /* Amount of spaces to indent outputs   */
int    scnt;                 /* total # of structures */
int    sdone;                /* running count of structures displayed */
int    hcnt;                 /* hinge count per structure */
long   totalreallayercnt;    /* total boxes in the resultant matlab file */
FILE   *mytmp;
FILE   *tfp;
pivotaxis_ty pivt;
structinfoty s[500];

/* function prototyping */
void fout(char *msg);
void tout(char *msg);
void indent( int inc, char b);
int findrotatedir( pivotaxis_ty p, geom_info_ty g);
void structureFirstPass();
void structureSecondPass();
void resequence();
void formlinkages();
void findneighbors();
void outputstats();
void drawthisstruct ( int m );

extern FILE *gfopen (char *f, char *m);

/**********************************************************/
/**********************************************************/

int processdata()
{
  FILE  *inf;

  inf   = gfopen( POSTMASONFILE, "r" );
  mytmp = gfopen( TMPFILE1, "w" );
  tfp   = tmpfile();

  scnt = 1;
  geom1st = TRUE;
  indentlevel = 0;
  totalreallayercnt = 0;
  hcnt = 0;

  while ( !feof(inf)) {

    (void) fgets(tmpstr, 80, inf);

    if ( strlen( (char *) tmpstr ) > 1 ){
      (void) structureFirstPass();
    }
    else {
      /* when there is a blank line in the mason output file, it */
      /* indicates the end of a structure, run structureSecondPass */
      /* for additional processing */
      (void) structureSecondPass();
    }
  }

  fclose(inf);
  fclose(mytmp);

  (void) findneighbors();   /* find connectivity info for each structure */

  if (dolinkage) {
     (void) formlinkages(); /* form linkages with the connectivity info */
  }

  (void) outputstats();     /* print out some pertinent information */
  (void) resequence();      /*  */

  return 0;
}
/**********************************************************/

void structureFirstPass()
{
  int   a, b, c, d, e, f; /* coordinates of each box */

  /* write everything read back into a tmp file */
  fprintf(tfp, "%s", tmpstr);

  /* format:  xmin xmax ymin ymax zmin zmax */
  sscanf(tmpstr, "%d %d %d %d %d %d", &a,&b,&c,&d,&e,&f);

  s[scnt].geom.min_a = ( geom1st ? a :  MIN(s[scnt].geom.min_a, a));
  s[scnt].geom.max_b = ( geom1st ? b :  MAX(s[scnt].geom.max_b, b));
  s[scnt].geom.min_c = ( geom1st ? c :  MIN(s[scnt].geom.min_c, c));
  s[scnt].geom.max_d = ( geom1st ? d :  MAX(s[scnt].geom.max_d, d));
  geom1st = FALSE;

  if (e==0){ 
    s[scnt].anchored = TRUE;
  }
  else if ((e==1 && f==4) || (e==5 && f==8)){

    if (e==1 && f==4) {
      s[scnt].hcoord[hcnt].matchpsuedo=1;
      s[scnt].hinged = TRUE;
    }

    if (e==5 && f==8){
      s[scnt].hcoord[hcnt].matchpsuedo=2;
    }

    s[scnt].hcoord[hcnt].a=a;
    s[scnt].hcoord[hcnt].b=b;
    s[scnt].hcoord[hcnt].c=c;
    s[scnt].hcoord[hcnt].d=d;
    s[scnt].hingecnt = hcnt;
    hcnt++;
  }
  else if (e==5 && f==9) {
    s[scnt].reflector.on = 1;
    s[scnt].reflector.a = a;
    s[scnt].reflector.b = b;
    s[scnt].reflector.c = c;
    s[scnt].reflector.d = d;
    s[scnt].reflector.e = e;
    s[scnt].reflector.f = f;
  }

  return;
}

/**********************************************************/
void structureSecondPass()
{
  int   a, b, c, d, e, f;  /* min/max of each box */
  int   reallayer;         /* indicator for whether a layer is pseudo layer */
 
  rewind(tfp);

  if (s[scnt].anchored == TRUE)
    s[scnt].hinged == FALSE;

  sprintf(tmpstr,"# structure %d start --------------------\n{\n", scnt);
  tout( tmpstr );

  /* format:  xmin xmax ymin ymax zmin zmax */
  while ( fscanf(tfp, "%d %d %d %d %d %d", &a,&b,&c,&d,&e,&f) == 6) {

    reallayer = ((e==1 && f==4)||(e==5 && f==8)||(e==5 && f==9)? FALSE : TRUE);

    if ( reallayer ) {
      s[scnt].lcnt++;
      totalreallayercnt++;
      sprintf(tmpstr,"  %.1f %.1f %.1f %.1f %.1f %.1f\n", 
                     (float)(b-a), (float)(d-c), (float)(f-e), 
                     (float)(b+a)/2., (float)(d+c)/2., (float)(f+e)/2.);
      tout(tmpstr);
    }
  }

  sprintf(tmpstr,"}\n# structure %d end --------------------\n\n", scnt);
  tout(tmpstr);

  fclose(tfp);
  tfp = tmpfile();
  scnt++;
  hcnt = 0;
  s[scnt].hinged = FALSE;
  s[scnt].anchored = FALSE;
  geom1st = TRUE;

  return;
}

/**********************************************************/
void outputstats()
{
  int i, j;

  if ( !printsummary )
    return;

  sprintf(tmpstr, "# ---------------------------------------\n");
  fout(tmpstr);
  sprintf(tmpstr, "# total structures found: %d\n", scnt-1);
  fout(tmpstr);
  sprintf(tmpstr, "# total boxes: %d\n", totalreallayercnt);
  fout(tmpstr);
  sprintf(tmpstr, "#\n");
  fout(tmpstr);

  sprintf(tmpstr, "# neighbor summary :\n");
  fout(tmpstr);
  for (i=1; i<scnt; i++){ 
    sprintf(tmpstr, "# struct %d:\t%d neighbor(s): ", i, s[i].ncnt );
    fout (tmpstr);
    for (j=0; j<10; j++) {
      if (s[i].narray[j] == 0 ) {
        sprintf(tmpstr, "\n");
        j=10;
      } 
      else {
        sprintf(tmpstr, "%d ", s[i].narray[j]);
      }
      fout (tmpstr);
    }
  }

  if (dolinkage) {
    sprintf(tmpstr, "#\n# linkage summary :\n");
    fout(tmpstr);
    for (i=1; i<scnt; i++){ 
      sprintf(tmpstr, "# struct %d:\thingecnt: %d\tlinked-> %d\tlinkage(s): ", 
                      i, s[i].hingecnt, s[i].linkedto );
      fout (tmpstr);
      if ( s[i].linkedto ) {
        for (j=0; j<s[i].linkagecnt; j++) {
          sprintf(tmpstr, "%d ", s[i].linkage[j]);
          fout (tmpstr);
        }
      }
      fout ("\n");
    }
  }

  sprintf(tmpstr, "# ---------------------------------------\n");
  fout(tmpstr);
  return;
}
/**********************************************************/
int nextattachedto ( int from, int i )
{
  int j;
  int found;

  if ( s[i].anchored)
    return 2;

  //printf ("ncnt: %d \n",s[i].ncnt);
  if (s[i].ncnt == 1)
    return 0;

  found =0; 
  for (j=0; j < s[i].ncnt; j++) {

    //printf ("i,j: %d %d\n",i,j);
    if  (s[i].narray[j] != from) found = nextattachedto (i,  s[i].narray[j]);

    if (found) {
      j = s[i].ncnt +1;  /* exit for-j loop */
      found =1;
    }
  }

  return found; 
}

/**********************************************************/
void formlinkages()
{
  int i, j, k;
  int found; /* found the attached to in the neighbor structure */ 

  for (i=1; i<scnt; i++) { 

    //printf ("%s(%d) struct: %d\n", FPOS, i );
    //printf ("hinged: %d\n", s[i].hinged);
    //printf ("ncnt:   %d\n", s[i].ncnt);

    /* start off by saying all is not attached */
    s[i].linkedto = -1;

    if ( !s[i].anchored ) {
      if (s[i].ncnt == 1 ) {
        s[i].linkedto = s[i].narray[0]; 
      }
      else {
        for (j=0; j < s[i].ncnt; j++) {
          //printf ("j: %d, narray[%d]: %d \n", j, j ,s[i].narray[j]);
          found = nextattachedto ( i, s[i].narray[j] );
          if (found) {
            //s[i].linkedto = (found==2 ? 0: s[i].narray[j]);
            s[i].linkedto = s[i].narray[j];
            j = s[i].ncnt +1;  /* exit for-j loop */
          }
        } 
      } 
    } 
  }

  for (i=1; i<scnt; i++) { 
    j=0;
    k=i;
    s[i].linkage[j++]= i;
    while ( s[k].linkedto != -1 ){
      s[i].linkage[j++] = s[k].linkedto;
      k = s[k].linkedto;
    } 
    s[i].linkagecnt = j-1;
  }


  return;
}

/**********************************************************/

void findneighbors()
{
  int i, j, k, n, nct;

  for (i=1; i<scnt; i++){
    nct = 0;
    for (j=0; j<=s[i].hingecnt; j++){
      for (k=1; k<scnt; k++){
        for (n=0; n<=s[k].hingecnt; n++){
          if ( (s[i].hcoord[j].matchpsuedo ==1 &&  
                s[k].hcoord[n].matchpsuedo ==2) ||
               (s[i].hcoord[j].matchpsuedo ==2 &&  
                s[k].hcoord[n].matchpsuedo ==1)) {

            if (s[i].hcoord[j].a == s[k].hcoord[n].a &&
                s[i].hcoord[j].b == s[k].hcoord[n].b &&
                s[i].hcoord[j].c == s[k].hcoord[n].c &&
                s[i].hcoord[j].d == s[k].hcoord[n].d )  {

              nct = nodupicateadd(nct, i, k);
              n=s[k].hingecnt+1;  /* get out of the inner loop */
              k=scnt;             /* get out of the outer loop */
            }
          }
        } 
      } 
    }
 
    s[i].ncnt = nct;
  } 

  return;
}

/**********************************************************/
int nodupicateadd( int nct, int i, int k)
{
  int w, flag;

  flag=0;
  for (w=0; w<=nct; w++){
    if (s[i].narray[w] == k){ flag=1;}
  }

  /* not duplicated and not self */
  if (!flag && (k != i)) {
    s[i].narray[nct] = k;
    nct++;
  }
  return nct;
}

/**********************************************************/
void nextlevel ( int m )
{
  int branch, i, j;

  branch = s[m].ncnt;

  if (branch==1 && s[m].hinged){
    drawthisstruct ( s[m].narray[0] );
  }
  else { 
    for (i=0; i<branch; i++){
      j=s[m].narray[i];
      if ( !s[j].done && s[j].hinged){
        drawthisstruct ( s[m].narray[i]);
        nextlevel (s[m].narray[i]);
      }
    }
  }
  return;
}
/**********************************************************/
void generaterotateinfo ( int m )
{
  int i, j, k, h; 
  int a, b, c, d;
  short xlongerthany;

  i = s[m].linkedto;  /* i is the struct being attached to */ 

  h =0;

  /* find the hinge that the current struct is attached to */  
  for (j=0; j<=s[i].hingecnt; j++){
    for (k=0; k<=s[m].hingecnt; k++){
      if ( (s[i].hcoord[j].matchpsuedo ==1  &&  
            s[m].hcoord[k].matchpsuedo ==2) ||
           (s[i].hcoord[j].matchpsuedo ==2  &&  
            s[m].hcoord[k].matchpsuedo ==1)) {

        if ( (s[i].hcoord[j].a == s[m].hcoord[k].a &&
              s[i].hcoord[j].b == s[m].hcoord[k].b &&
              s[i].hcoord[j].c == s[m].hcoord[k].c &&
              s[i].hcoord[j].d == s[m].hcoord[k].d )) {

           h = k;   /* h is now the hinge doing the attaching */
           k = s[m].hingecnt +1; /* exit inner loop */
           j = s[i].hingecnt +1; /* exit outer loop */

        }
      }
    } 
  } 

  a = s[m].hcoord[h].a;
  b = s[m].hcoord[h].b;
  c = s[m].hcoord[h].c;
  d = s[m].hcoord[h].d;
 
  xlongerthany = ((b-a)>(d-c) ? 1 : 0);
  pivt.pivot = ( xlongerthany ? XPIVOT    : YPIVOT);
  pivt.x     = ( xlongerthany ? 0.0       : (a+b)/2.0);;
  pivt.y     = ( xlongerthany ? (d+c)/2.0 : 0.0);;
  pivt.z     = 2.0;

#if 0
  rotatedir = findrotatedir( s[m].piv1,  s[m].geom );
#endif

  return; 
}

/**********************************************************/
void outputstructinfo ( int m )
{
  char memberstart[8];
  char pvtstr[10];
  int  i;
  int  rotatedir;

  sprintf (tmpstr,"  anchor: %s\n", s[m].anchored ? "true" : "false");
  fout (tmpstr);
  if (!s[m].anchored) {
    for (i=0; i<s[m].linkagecnt; i++) {
      sprintf (memberstart,"%s",  i==0 ? "start" : "member");
      sprintf (tmpstr,"  link %d %s\n", s[m].linkage[i], memberstart);
      fout (tmpstr);
    }

    generaterotateinfo ( m );

    switch (pivt.pivot) {
      case 8:  sprintf (pvtstr, "%s", "+x" );
               break;
      case 4:  sprintf (pvtstr, "%s", "+y" );
               break;
      default: sprintf (pvtstr, "%s", "+z" );
               break;
    }
  
    sprintf(tmpstr,"  rotate: %s %.2f %.2f %.2f\n", 
	            pvtstr, pivt.x, pivt.y, pivt.z);
    fout( tmpstr );
  } 

  if ( s[m].reflector.on ){
    fout("\n  reflect start\n");
    sprintf(tmpstr,"  %.1f %.1f %.1f %.1f %.1f %.1f\n", 
                   (float)(s[m].reflector.b-s[m].reflector.a),
                   (float)(s[m].reflector.d-s[m].reflector.c),
                    1.5, /* hard code the thickness for now */
                   (float)(s[m].reflector.b+s[m].reflector.a)/2.,
                   (float)(s[m].reflector.d+s[m].reflector.c)/2.,
                   (float)(s[m].reflector.e+2));
    fout(tmpstr);
    fout("  reflect end\n");
  }

  sprintf (tmpstr, "\n");
 
}

/**********************************************************/
void drawthisstruct ( int m )
{
  char s1[20], s2[20], s3[20], s4[20], s5[20];
  short exitloop, spitout;
  int i;
  static int structcounter=1;

  if (!s[m].done){
    exitloop = 0;
    spitout = 0;

    while (!feof(mytmp) && !exitloop){
      (void) fgets(tmpstr,80,mytmp);

       if (sscanf (tmpstr, "%s %s %s %s %s", &s1, &s2, &s3, &s4, &s5) == 5 ){
    
        if(!strcmp(s1, "#") && !strcmp(s4, "start") && atoi(s3)==m ){

          (void) fgets(tmpstr,80,mytmp); /* get the beginning bracket */
	  sprintf (tmpstr, "\n# structure %d start -----------------\n",
                            structcounter);
	  fout(tmpstr);

	  sprintf ( tmpstr,"# (struct %d in the neighbor summary.)\n{\n", m);
          printsummary ? fout(tmpstr) : fout ("{\n" );

          outputstructinfo (m);
          spitout = 1;
        }
        if(!strcmp(s1, "#") && !strcmp(s4, "end") && atoi(s3)==m ){
	  sprintf (tmpstr, "# structure %d end -----------------\n",
                            structcounter);
	  fout(tmpstr);
          spitout = 0;
          exitloop = 1;
          structcounter++;
        }
      }

      if (spitout){ fout(tmpstr);}
    }

    rewind(mytmp);

    s[m].done = 1;
    sdone++;
  }

  return;
}
/**********************************************************/
/* re-sequencing the structures */
void resequence()
{
  int base[100];
  int donotskipthis; 
  int m, g, h;
  int bcnt;

  mytmp = fopen( TMPFILE1, "r");
  indentlevel = 0;
  bcnt=0;
  sdone=0;

  if (doresequence) {
    /* draw the anchored structs */
    for (m=1; m<=scnt; m++){ 
      if (s[m].anchored) {
        drawthisstruct ( m );
      }
    }

    for (m=1; m<=scnt; m++){ 
      if (!s[m].hinged) {
        donotskipthis = 1;

        for (g=0; g < bcnt; g++){
          for (h=0; h < s[m].ncnt; h++){
            if ( base[g] == s[m].narray[h]){ 
              donotskipthis =0; 
            }
          } 
        }

        if (donotskipthis) {
          base[bcnt] = s[m].narray[0]; 
          bcnt++;
          nextlevel( m );  /* note:  m is the current structure */
        }
      }
    }
  }

  /* for doresequence - draw the reminder objects */
  /* otherwise draw all */
  for (m=1; m<=scnt; m++){ 
    drawthisstruct ( m );
  }
 

  fclose (mytmp);
  return;                                     
}

/**********************************************************/
int findrotatedir( pivotaxis_ty p, geom_info_ty g)
{
  float min, max;
  int sense;
  
  /* look at pivoting about x-axis */
  if ( p.pivot == XPIVOT ){
    min = (float)(g.min_c); 
    max = (float)(g.max_d); 
    sense = (abs(p.y-min) > abs(max-p.y) ? -1 : 1); 
  }

  /* look at pivoting about y-axis */
  else if ( p.pivot == YPIVOT ){
    min = (float)(g.min_a); 
    max = (float)(g.max_b); 
    sense = (abs(p.x-min) > abs(max-p.x) ? 1 : -1);
  }
  else {
    sense = 1;  /* should never get here */
  }
  return sense;
}

/**********************************************************/
void fout(char *msg)  /* Outputs an indented string   */
{
  int i;
  for ( i=0; i<indentlevel; i++ ) fprintf(OUTPUTTO,"   ");
  fprintf(OUTPUTTO, msg);
}

/**********************************************************/
void tout(char *msg) /* Output to a temp file */
{
  int i;
  for ( i=0; i<indentlevel; i++ ) fprintf(mytmp,"   ");
  fprintf(mytmp,msg);
}

/**********************************************************/
void indent( int inc, char b)
{
  (char) *(braces+indentlevel) = b;   
  if (inc) { indentlevel++; }
  else     { indentlevel--; }
}

