/*
 *--------------------------------------------------------------------------
 * Copyright Notice:
 *
 * Copyright 1996, 1997, 1998, 1999, 2000 The Regents of the University of 
 * California.  All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the 
 * above copyright notice and the following three paragraphs appear in all 
 * copies.
 *
 * Permission to incorporate this software into commercial products may be
 * obtained from the Office of Intellectual Property Administration, Pete 
 * Ueberoth Building, UCLA, Los Angeles, CA 90095-1594.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGES.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATIONS
 * TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 * 
 *--------------------------------------------------------------------------
 */
/*
  Cif2mat2
 
  This program converts cif to mat3d format.  It can can handle:
	B - Boxes
	DS - Scaling, Name, and Cell definition
	DF - End of definition
	C # T # # - translation and cell instance

  History:
  1.0 - got the basic code working

*/

#include "stdio.h"
#include "time.h"

#define CIF2MAT_MAIN
#include "cif2mat2.h"
#undef  CIF2MAT_MAIN

void MatHeader();
void MatFooter();
void usage ();
void cmdline( int argc, char **argv);
void init();
void cleanup();
FILE *gfopen (char *fn, char *mode);

extern void processcif();
extern void processlayers();
extern void blockseparation();
extern void minimizeboxes();
extern void processdata();

/**********************************************************/
/**********************************************************/
int main( int argc, char **argv)
{
  (void) init ();

  (void) cmdline (argc, argv);

  in  = gfopen( infilename, "rb");
  if (strlen(outfilename) ){
    out = gfopen( outfilename, "w");
    tofile = 1;
  }

  (void) processcif();       /* read cif file and separate cif layers */
  (void) processlayers();    /* layer mask, boolean operation, etc. */
  (void) blockseparation();  /* separate spatially disjointed blocks into
                                strucutures */
  (void) minimizeboxes();    /* attempts to minimize the number of boxes
                                within each structure */
  (void) MatHeader();
  (void) processdata();
  (void) MatFooter();

  if (in)  fclose(in);
  if (out) fclose(out);

  (void) cleanup();

}

/**********************************************************/
void MatFooter()
{
  fprintf(OUTPUTTO,"#\n# end of mat3d file\n");
  return;
}

/**********************************************************/
void MatHeader()
{
  time_t now;
  char tmpstr[100];

  now = time(NULL);
  strftime(tmpstr,100,"# generated on %d %B %Y, %H:%M.\n",localtime(&now)); 

  fprintf(OUTPUTTO,"#\n# cif input file: %s\n#\n", infilename);
  fprintf(OUTPUTTO,"# Cif file convert to mat3d format\n");
  fprintf(OUTPUTTO,"# Use mat3d matlab script to render and manipulate it.\n");
  fprintf(OUTPUTTO,"# This is a generated data file, edit at own risk.\n");
  fprintf(OUTPUTTO,"#\n");
  fprintf(OUTPUTTO,"# Please direct questions/comments for mat3d to: \n");
  fprintf(OUTPUTTO,"#   Nanping R. Lo\n");
  fprintf(OUTPUTTO,"#   nanping@ee.ucla.edu\n#\n");
  fprintf(OUTPUTTO,"%s", tmpstr);
  fprintf(OUTPUTTO,"#\n");
  return;
}

/**********************************************************/
void usage ()
{
  printf ("usage: %s [-h][-s][-l yes|no][-r yes|no][-o matname] cifname\n", 
           ProgramName);
  printf ("  -o output mat3d file name.\n");
  printf ("  -s generate structure summary in output (default no).\n");
  printf ("  -r [yes|no] attempts to group structures (default no).\n");
  printf ("  -l [yes|no] whether to form linkage info (default yes).\n");
  printf ("  -h help (this screen).\n\n");
  exit (0);
}

/**********************************************************/
void cmdline( int argc, char **argv)
{
  int i;

  if (argc == 1){ usage (); }

  ProgramName = argv[0];

  for (i = 1; i < argc; i++) {
    if (*argv[i] == '-') {
      switch (*(argv[i]+1))
      {
         case 's':
           printsummary = 1;
           break;

         case 'h':
           usage();
           break;

         case 'r':
           doresequence = (!strcmp(argv[i+1],"yes") ? 1 : 0);
           i++;
           break;

         case 'l':
           dolinkage = (!strcmp(argv[i+1],"no") ? 0 : 1);
           i++;
           break;

         case 'o':
           strcpy(outfilename, argv[i+1]);
           i++;
           break;

         default:
           break;
      }
    } 
    else {
      strcpy(infilename, argv[argc-1]);
    }
  } 
  return;
}

/**********************************************************/
void init()
{
  printsummary = 0;
  doresequence = 0;
  dolinkage = 1;
  tofile = 0;
  return;
}

/**********************************************************/
void cleanup()
{
  unlink ( TMPFILE1 );
  unlink ( TMPFILE2 );
  unlink ( PREMASONFILE );
  unlink ( POSTMASONFILE );
  unlink ( MINIMIZEDMASONFILE );
  return;
}

/**********************************************************/
FILE *gfopen (char *fn, char *mode)
{
  FILE *fp;

  if((fp = fopen(fn, mode)) == NULL) {
    fprintf (stderr, "Error opening file %s\n", fn);
    exit(1);
  }
  return fp;
}


