/*
 *--------------------------------------------------------------------------
 * Copyright Notice:
 *
 * Copyright 1996, 1997, 1998, 1999, 2000 The Regents of the University of 
 * California.  All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the 
 * above copyright notice and the following three paragraphs appear in all 
 * copies.
 *
 * Permission to incorporate this software into commercial products may be
 * obtained from the Office of Intellectual Property Administration, Pete 
 * Ueberoth Building, UCLA, Los Angeles, CA 90095-1594.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGES.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATIONS
 * TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 * 
 *--------------------------------------------------------------------------
 */
/*
  decomposepoly.c 
  
  part of the cif2mat program.
  this module serves to decompose the polygon output of layer boolean 
  operations into trapezoids  (in this application with manhattanized 
  geometries, the trapezoids are rectangles).  it is most likely that 
  this decomposition path will not be taken.  this portion of the code 
  has not been tested too much, and might be susceptible to bugs.

*/

#include "trapezoid.h"

#define MAXPOLY 250 

/**********************************************************/
/**********************************************************/

void decomposepoly(int n, int x[], int y[])
{
	//int n;
	//int x[MAXPOLY];
	//int y[MAXPOLY];
	TGIndex *traplist;

        printf ("cnt:%d (%d %d)\n", n, x[0], y[0]);

	//readPolyFromFile("poly.dat",&n,x,y);
	traplist = trapezoidalDecomposition(n,x,y);
	//displayTrapList(traplist);
        fprintf (stderr, "%s(%d)\n", __FILE__,__LINE__);
}

/**********************************************************/
static void displayTrapezium(Trapezium *trap)
{
	int y, x1,y1,x2,y2,x3,x4;
	double dy, dxdy1, dxdy2, xleft, xright;
        static cnt;

	x1 = ROUND(trap->x1);
	y1 = trap->y1;
	x2 = ROUND(trap->x2);
	y2 = trap->y2;
	x3 = ROUND(trap->x3);
	x4 = ROUND(trap->x4);

	dy = y2-y1;
	dxdy1 = (trap->x2-trap->x1)/dy;
	dxdy2 = (trap->x3-trap->x4)/dy;

        printf ("trapezoid %d\n", cnt++);	
        printf ("(%d %d) (%d %d) (%d %d) (%d %d) \n", 
                 x1,y1,x2,y1,x3,y2,x4,y2);
}

/**********************************************************/
static void displayTrapList(TGIndex *traplist)
{
	register int i;
	GIndex *gind;
	Trapezium *trap;

	gind = traplist->first;
	while(gind){
		trap = (Trapezium *)gind->object;
		displayTrapezium(trap);
		gind = gind->next;
	}
}

/**********************************************************/
void readPolyFromFile(char *filename, int *n, int x[], int y[])
{
	FILE *fp;
	int i;

	fp = fopen(filename,"r");
	fscanf(fp,"%d",n);
	for(i=0;i<*n;++i) fscanf(fp,"%d %d",&x[i],&y[i]);
	fclose(fp);
}

