/*
 *--------------------------------------------------------------------------
 * Copyright Notice:
 *
 * Copyright 1996, 1997, 1998, 1999, 2000 The Regents of the University of 
 * California.  All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the 
 * above copyright notice and the following three paragraphs appear in all 
 * copies.
 *
 * Permission to incorporate this software into commercial products may be
 * obtained from the Office of Intellectual Property Administration, Pete 
 * Ueberoth Building, UCLA, Los Angeles, CA 90095-1594.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGES.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATIONS
 * TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 * 
 *--------------------------------------------------------------------------
 */
/* 
  processlayers.c
*/

#include "stdio.h"

#include "gpc.h"
#include "cif2mat2.h"

#define EMPTYLAYERFILE "empty.gpc"
 
struct  ListItem { 
  char   Item[20];  
  struct ListItem *Next;
};


extern FILE *gfopen (char *f, char *m);
extern void decomposepoly(int cnt, int x[], int y[]);

void create_empty_layerfile( void );
FILE *openlayerfile (char *f, char *m);
void addlayertolist ( char *layerfilename );
void removelayerlist(void); 
void layers_op( gpc_op operator, char *sf, char *cf, char *rf);
void layercopy(char *in, char *out);
void layer_grow(char *in, char *out, int amount);
void recombinelayers( char **rlayerlist );
void cleanuplayers (void);

/* this is the layerlist that keeps track of all the layers in
   the cif file, as well as the temporary layers created during
   layer boolean operation. initially empty */
struct ListItem *layerlist = 0;    

/**********************************************************/
/**********************************************************/
void processlayers()
{

  /* only the layers listed in this list will be process
     in the block separation routines. */
  char *recombinelayerlist[] = 
  {"COL",   /* c2s */
   "COT",   /* c11 */
   "COF",   /* c1s */
   "mtl",   /* metal */
   "refl",  /* reflector layer */
   "pvtl", 
   "pvth", 
   "ply1", 
   "p2lo",
   "p2hi",
   "ply2",
    NULL};

  (void) create_empty_layerfile();

  /* pvt1 = (grow(c2s,5) - c2s) *ply2 *ply1 */
  (void) layer_grow("COL","c2st", 5); 
  (void) layers_op( GPC_DIFF, "c2st", "COL", "tmp5");
  (void) layers_op( GPC_INT,  "tmp5", "CPT", "tmp6");
  (void) layers_op( GPC_INT,  "tmp6", "CPS", "pvt1");

  /* pvt2 = (ply1 * ply2 * bpmtl) - (c21 * ply1 * ply2 * bpmtl) - mtl */
  (void) layers_op( GPC_INT, "CPS", "CPT", "tmp7");
  (void) layers_op( GPC_INT, "tmp7","CMB", "tmp8");
  (void) layers_op( GPC_INT, "COT", "tmp8","tmp9");
  (void) layers_op( GPC_DIFF,"tmp8","tmp9","tmp10");
  (void) layers_op( GPC_DIFF,"tmp10","CCM","pvt2");

  /* pvtl = pvt1 + pvt2 */ 
  (void) layers_op( GPC_UNION,"pvt1","pvt2","pvtl");
   
  /* pvth to be processed later with ply2 in processstructures */
  /* pvtl tp be processed later with ply1 in processstructures */
  /* pvth = pvtl */
  (void) layercopy ("pvtl", "pvth");

  /* mtl = mtl - notmtl */
  (void) layers_op( GPC_DIFF, "CCM", "CHM", "mtl");

  /* refl = bpmtl * mtl */
  (void) layers_op( GPC_INT, "CCM", "CMB", "refl" );

  /* ply1 = ply1 - notp1 */
  (void) layers_op( GPC_DIFF,  "CPS", "CHO", "ply1" );

  /* ply2 = ply2 - notp2 */
  (void) layers_op( GPC_DIFF,  "CPT", "CHT", "p2tmp" );

  (void) layer_grow("ply1","p11", 2); 
  (void) layer_grow("ply1","p12", 4); 
  (void) layer_grow("ply1","p13", 6); 

  /* p2hi = p12 * ply2 */
  (void) layers_op( GPC_INT, "p12","p2tmp","p2hi" );

  /* p2lo = p13*ply2 - p11*ply2 - p12*p13*ply2 */
  (void) layers_op( GPC_INT,  "p13",  "p2tmp", "tmp1" );
  (void) layers_op( GPC_INT,  "p11",  "p2tmp", "tmp2" );
  (void) layers_op( GPC_INT,  "p13",  "p2hi",  "tmp3" );
  (void) layers_op( GPC_DIFF, "tmp1", "tmp2",  "tmp4" );
  (void) layers_op( GPC_DIFF, "tmp4", "tmp3",  "p2lo" );

  /* ply2 = ply2 - p2hi */
  (void) layers_op( GPC_DIFF, "p2tmp", "p2hi", "ply2" );


#if CIF2MAT_DEBUG /* gpc2m3d is for debug purposes */
  gpc2m3d("ply1",2.0,4.0);
  gpc2m3d("p2lo",4.0,6.0);
  gpc2m3d("p2hi",5.0,7.0);
  gpc2m3d("ply2",2.0,4.0);

  gpc2m3d("p2tmp",2.0,4.0);
  gpc2m3d("p11",2.0,4.0);
  gpc2m3d("p12",2.0,4.0);
  gpc2m3d("p13",2.0,4.0);
#endif

  (void) recombinelayers( recombinelayerlist );

  /* get rid of the layer files */
  (void) cleanuplayers ();

  return;
}

/**********************************************************/

void layer_union_from_file(char *filename, int read_hole, 
                           gpc_polygon *p)
{
  gpc_polygon tmp;
  int c, v, ccnt, vcnt;
  int hole;
  double x, y;
  FILE  *tfp, *fp;

  fp = gfopen ( filename, "r");
  fscanf(fp, "%d", &ccnt);

  for (c= 0; c < ccnt; c++)
  {
    tfp = gfopen ( TMPFILE2, "w");

    fscanf (fp,  "%d", &vcnt);

    fprintf(tfp, "1\n");
    fprintf(tfp, "%d\n", vcnt);

    if (read_hole) fscanf(fp, "%d", &hole);

    for (v= 0; v < vcnt; v++) {
      fscanf(fp,   "%lf %lf", &x, &y);
      fprintf(tfp, "%.5f %.5f\n", x, y);
    }
    fclose (tfp);

    tfp = gfopen ( TMPFILE2, "r");
    gpc_read_polygon (tfp, 0, &tmp);
    gpc_polygon_clip(GPC_UNION, p, &tmp, p);

    fclose (tfp);
  }
  fclose (fp);
}

/**********************************************************/
void layercopy(char *infile, char *outfile)
{
  FILE *inf, *outf;
  char ifn[48], ofn[48];
  char buf[100];

 if (!strcmp (infile, outfile)){
     fprintf (stderr, "%s(%d): Error: output specification same as input.\n", FPOS);
     exit(1);
  }

  strcpy(ifn, infile);
  strcpy(ofn, outfile);
  strcat(ifn, ".gpc");
  strcat(ofn, ".gpc");

  inf  = gfopen(ifn, "r");
  outf = openlayerfile(ofn, "w");

  while ( fgets(buf, 100, inf) != NULL)
     fprintf (outf, "%s", buf);

  fclose (inf);
  fclose (outf);

  return;
}

/**********************************************************/

void layer_grow(char *infile, char *outfile, int dist)
{
  gpc_polygon  subject;
  gpc_tristrip result;
 
  FILE *inf, *outf, *tfp;
  char ifn[48], ofn[48];
  int  i, j, bcnt, numv;
  float x,y,xhi,xlo,yhi,ylo;
  char buf[100];

  if (!strcmp (infile, outfile)){
     fprintf (stderr, "%s(%d): Error: output specification same as input.\n", 
                      FPOS);
     exit(1);
  }

  strcpy(ifn, infile);
  strcpy(ofn, outfile);
  strcat(ifn, ".gpc");
  strcat(ofn, ".gpc");

  if((inf = fopen(ifn, "r")) == NULL) 
    return; /* nothing to grow or shrink */

  //inf = gfopen(ifn, "r");
  tfp = gfopen(TMPFILE1, "w");

  fscanf  (inf,  "%d", &bcnt);
  fprintf (tfp, "%d\n", bcnt);
  for (i=0; i<bcnt; i++) {
    fscanf  (inf, "%d", &numv);
    fprintf (tfp, "%d\n", numv);
    if ( numv == 4 ) {
      for (j=0; j<4; j++) {
        fscanf  (inf, "%f %f", &x,&y);
        if (j==0) {
          xhi = x; xlo = x;
          yhi = y; ylo = y; 
        }
        else {
          xhi = MAX (xhi, x);
          xlo = MIN (xlo, x);
          yhi = MAX (yhi, y);
          ylo = MIN (ylo, y);
        }
      } 
      xhi = xhi + dist;
      xlo = xlo - dist;
      yhi = yhi + dist;
      ylo = ylo - dist;
      fprintf(tfp,"%.2f %.2f\n", xlo,ylo);
      fprintf(tfp,"%.2f %.2f\n", xhi,ylo);
      fprintf(tfp,"%.2f %.2f\n", xhi,yhi);
      fprintf(tfp,"%.2f %.2f\n", xlo,yhi);
    }
    else {
      fprintf (stderr, "%s(%d): Error: nuber of vertices != 4.\n", FPOS);
      for (j=0; j<bcnt; j++) { fscanf  (inf, "%f %f", &x,&y);}
    }

  } 
  fclose(inf);
  fclose(tfp);

  /* 
  the grow process creates many overlapping layers. however, 
  since the gpc library does not handle the overlapping layers 
  well (it treats the overlapped portion as "hole" layer), so the
  following steps are neccessary to work around this problem.
  */

  inf = gfopen ( EMPTYLAYERFILE, "r");
  outf = openlayerfile(ofn, "w");

  gpc_read_polygon(inf, 0, &subject);
  layer_union_from_file (TMPFILE1, 0, &subject);
  gpc_polygon_to_tristrip (&subject, &result);
  gpc_write_tristrip(outf, &result);
    
  gpc_free_polygon(&subject);
  gpc_free_tristrip(&result);

  fclose(inf);
  fclose(outf);
}

/**********************************************************/

#define MAXVERTICES 250

void gpc2m3d( char *fn, float zlo, float zhi)
{
  FILE *xin, *xout;
  int i, j, bcnt, numv;
  float x,y,xhi,xlo,yhi,ylo;
  int vx[MAXVERTICES], vy[MAXVERTICES];
  char infname[24], outfname[24];

  strcpy(infname,  fn);
  strcpy(outfname, fn);
  strcat(infname,  ".gpc");
  strcat(outfname, ".m3d");
  
  xin=  gfopen(infname, "r");
  xout= gfopen(outfname, "w");
  fscanf  (xin, "%d", &bcnt);
  fprintf (xout, "{\n  anchor: false\n"); 
  for (i=0; i<bcnt; i++) {
    fscanf  (xin, "%d", &numv);
    if ( numv == 4 ) {
      for (j=0; j<4; j++) {
        fscanf  (xin, "%f %f", &x,&y);
        if (j==0) {
          xhi = x; xlo = xhi;
          yhi = y; ylo = yhi; 
        }
        else {
          xhi = MAX (xhi, x);
          xlo = MIN (xlo, x);
          yhi = MAX (yhi, y);
          ylo = MIN (ylo, y);
        }
      } 
      fprintf(xout,"  %.1f %.1f %.1f %.1f %.1f %.1f\n",
                      ABS(xhi-xlo), ABS(yhi-ylo), ABS(zhi-zlo), 
                     (xhi+xlo)/2.0, (yhi+ylo)/2.0, (zhi+zlo)/2.0);
    }
    else { 
      /* number of vertices neq 4 */
      /* probably will never get here, but flag it if it does! */
      for (j=0; j<numv; j++) { 
         fscanf  (xin, "%f %f", &x,&y);
         vx[j]=(int)(x);
         vy[j]=(int)(y);
      }
      fprintf (stderr,"%s(%d): Error: %s-> verts = %d\n", FPOS, fn, numv);
      decomposepoly( numv,vx,vy );
    }

  } 
  fprintf (xout, "}\n"); 
  fclose(xin);
  fclose(xout);

}

/**********************************************************/

void create_empty_layerfile()
{
  FILE *e;

  e = openlayerfile ( EMPTYLAYERFILE, "w");
  fprintf ( e, "0\n");
  fclose ( e );
} 

/**********************************************************/

void layers_op( gpc_op operator, char *sf, char *cf, char *rf)
{
  static gpc_polygon subject, cliper;
  static gpc_tristrip result;
  FILE *subj, *clip, *res;
  char sfn[48], cfn[48], rfn[48];

  strcpy(sfn,  sf);
  strcpy(cfn,  cf);
  strcpy(rfn,  rf);
  strcat(sfn,  ".gpc");
  strcat(cfn,  ".gpc");
  strcat(rfn,  ".gpc");

  if((subj = fopen(sfn, "r")) == NULL) 
    subj = gfopen( EMPTYLAYERFILE, "r");

  if((clip = fopen(cfn, "r")) == NULL) 
    clip = gfopen( EMPTYLAYERFILE, "r");

  gpc_read_polygon(subj, 0, &subject);
  gpc_read_polygon(clip, 0, &cliper);
  
  gpc_tristrip_clip(operator, &subject, &cliper, &result);

  res = openlayerfile ( rfn, "w");
  gpc_write_tristrip(res, &result);
    
  gpc_free_polygon(&subject);
  gpc_free_polygon(&cliper);
  gpc_free_tristrip(&result);
          
  fclose(subj);
  fclose(clip);
  fclose(res);

  return;
}

/**********************************************************/

void cleanuplayers ()
{
  struct ListItem *p;

#ifndef CIF2MAT_DEBUG
  for ( p= layerlist; p != 0; p = p ->Next ) {  
    // printf("%s\n",p->Item );
    unlink( p->Item );
  }
#endif

  removelayerlist();
}

/**********************************************************/

void layer_height ( char *ll, int *lo, int *hi, int *thick )
{
  char l[5];
  int i;
  static int ox1=1, ox2=2, ply=2, ply1=2, ply2=2, metal=1, reflect=4, pivt=3;

  for (i=0; i< strlen(ll); i++) 
    l[i] = toupper (ll[i]);

  l[i] = '\0';

  *lo    = 0; 
  *thick = 0;

  if(!strcmp(l,"PLY1")){*lo = ox1;               *thick = ply1;}
  if(!strcmp(l,"PLY2")){*lo = ox1+ox2;           *thick = ply2;}
  if(!strcmp(l,"P2LO")){*lo = ox1+ox2+1;         *thick = ply1+ply2-2;} 
  if(!strcmp(l,"P2HI")){*lo = ox1+ox2+ply1;      *thick = ply2;}
  if(!strcmp(l,"C2SL")){*lo = 0;                 *thick = ply2;}
  if(!strcmp(l,"C2SH")){*lo = 0;                 *thick = ox1+ox2+ply2-1;}
  if(!strcmp(l,"C21L")){*lo = ox1+ply1;          *thick = ply2;} 
  if(!strcmp(l,"C21H")){*lo = ox1+ply1;          *thick = ox2+ply2-1;} 
  if(!strcmp(l,"COF" )){*lo = 0;                 *thick = ply2;}
  if(!strcmp(l,"COT" )){*lo = ox1;               *thick = ply1+ox2+ply2;}
  if(!strcmp(l,"COL" )){*lo = 0;                 *thick = ox1+ox2+ply1;}
  if(!strcmp(l,"CCM" )){*lo = ox1+ox2+ply1+ply2; *thick = metal;}
  if(!strcmp(l,"MTL" )){*lo = ox1+ox2+ply1+ply2; *thick = metal;}
  if(!strcmp(l,"PVTL")){*lo = ox1;               *thick = pivt;}
  if(!strcmp(l,"PVTH")){*lo = ox1+ox2+ply1;      *thick = pivt;}
  if(!strcmp(l,"REFL")){*lo = ox1+ox2+ply2;      *thick = reflect;} /*touch p2*/
  if(!strcmp(l,"BASE")){*lo = -2;                *thick = 1;}

  *hi  = *lo + *thick;
}

/**********************************************************/

void recombinelayers( char **layers)
{
  int i, j, bcnt, vcnt;
  int xhi, xlo, yhi, ylo, zhi, zlo, zthick;
  float x,y;
  int c=0;

  FILE *premason, *inf;
  char layerfile[48];

  premason = gfopen ( PREMASONFILE,"w");

  while (layers[c] != NULL ){
    sprintf (layerfile, "%s.gpc", layers[c]);
    //fprintf (stderr, "%d %s\n", c, layerfile);
    if ((inf = fopen( layerfile, "r")) != NULL) {

      layer_height ( layers[c], &zlo, &zhi, &zthick);

      fscanf  (inf, "%d", &bcnt);
      for (i=0; i<bcnt; i++) {
        fscanf  (inf, "%d", &vcnt);
        if (vcnt != 4) {
          fprintf (stderr,"%s(%d): number of vertices should be 4.\n", FPOS);
          for (j=0; j<vcnt; j++)  {
            fscanf  (inf, "%f %f", &x,&y);
          }
        }
        else {
          for (j=0; j<4; j++) { 
            fscanf  (inf, "%f %f", &x,&y);

            if (j==0) {
              xhi = (int)x; xlo = xhi;
              yhi = (int)y; ylo = yhi; 
            }
            else {
              xhi = MAX (xhi, (int)x);
              xlo = MIN (xlo, (int)x);
              yhi = MAX (yhi, (int)y);
              ylo = MIN (ylo, (int)y);
            }
          } 
          fprintf (premason,"%d %d %d %d %d %d\n", 
                            xlo,xhi,ylo,yhi,zlo,zhi);
        }
      }
      fclose (inf);
    }
    c++;
  }
  fclose (premason);
}

/**********************************************************/
FILE *openlayerfile (char *fn, char *mode)
{
  FILE *fp;
  fp = gfopen ( fn, mode );
  addlayertolist ( fn );
  return fp;
}

/**********************************************************/

void removelayerlist ()
{
  struct ListItem *p,*next;

  if ( layerlist== 0 ) {  return;  } 

  for(p = layerlist; p != 0 ; p = next )
  {
    next = p->Next; /* Save the next item position */
    free ( p );     /* release the item at p */
  }

  /* make sure that list points to 0 (empty list) */
  layerlist=0;  
  return;
}

/**********************************************************/
void addlayertolist ( char *layerfilename )
{
  struct ListItem *item,*last;

  item = (struct ListItem *) malloc(sizeof(struct ListItem));

  if ( item==0 ) { printf ("Malloc failed \n"); return; }

  strcpy(item->Item, layerfilename);
  item->Next = 0;

  if ( layerlist == 0 ) {  
    layerlist = item;
    return;
  }

  /* find the last item in the list.*/
  for( last = layerlist; last->Next != 0; last = last->Next )
  {} 

  /* append new item to last item */
  last ->Next = item;

  return;
}


