function cycle (state)
% CYCLE sequentially highlighting thru all structures in the current graph.
%  This is useful in identifyinf what is the nummerical assignment of each
%  structures in the graph.
%  CYCLE, by itself, highlights the 1st structure in the graph and pause
%  for user's keyboard input, then highlights the 2nd structure, and pause,
%  etc.., until all structures have been highlighted.
%  CYCLE (1), automatically sequence and highlight thru all structures in 
%  the graph.
%  CYCLE (2), same as CYCLE 1, except that the anchored structures are not 
%  highlighted.
%
%  For example,
%    CYCLE
%    CYCLE (2)
%
%  See also MAT3D, HILITE, SEL

% Copyright (c)1999, Nanping R. Lo nanping@ee.ucla.edu

if nargin == 0
   autocycle = 0;
else
   noanchored = 0;
   autocycle = 1;
   
   if state == 2
      noanchored = 1;
   end
end

global numstruct;
global istruct;

if (autocycle)
   for i=1:numstruct
      if ( noanchored & istruct(i).anchored == 1)
         % do nothing here
      else
        hiliteObject(i);
        tmpstr = sprintf ('M3D: structure %d hilited.', i);
        disp (tmpstr);     
        pause(1);
      end 
   end
else
   for i=1:numstruct
      hiliteObject(i);
      tmpstr = sprintf ('M3D: structure %d hilited. ''q'' to quit, other keys to continue... ', i);
      response = input(tmpstr,'s');
      if (~isempty(response))   
         if strcmp (response, 'q')
            break
         end
      end
   end
end

unhilite;

if (i ~= numstruct)
   tmpstr = sprintf ('M3D: cycled thru %d structures.', i);
else
   if (noanchored)
      tmpstr =sprintf ('M3d: done cycling thru non-anchored structures');
   else
      tmpstr = sprintf ('M3D: done cycling thru all %d structures...', i);
   end
end

disp (tmpstr);

%[x, y] = ginput(1)
