function m3rotate(t, a, p)
% M3ROTATE
% format 1: m3rotate (amount-of-rotation) 
% format 2: m3rotate (amount-of-rotation, rotate-direction, pivot-point, ) 
%
%  example: m3rotate(45)
%           m3rotate(45, [1 0 0], [20 10 2])  
%
%  See also MAT3D, M3TRANS, SEL

% Copyright (c)1999, Nanping R. Lo nanping@ee.ucla.edu


global selectedObj;
global numstruct;
global istruct;
global animate;
global m3debug;

n = selectedObj;

if (nargin == 1)
   theta  = t;
   axis   = istruct(n).rotatedir;
   %pivpt  = istruct(n).pivot;
elseif (nargin == 3)
   theta  = t;
   axis   = a;
   pivpt  = p;
else
   error('M3D: incorrect number of inputs.');
end

xpts = get (istruct(n).dirhdl, 'XData');
ypts = get (istruct(n).dirhdl, 'YData');
zpts = get (istruct(n).dirhdl, 'ZData');

if nargin ==1
  pivpt  = [xpts(1) ypts(1) zpts(1)];
  rotdir = [xpts(2)-xpts(1) ypts(2)-ypts(1) zpts(2)-zpts(1)];
else
  rotdir =a;   
end

if selectedObj == 0
   disp ('M3D error: no object selected for rotation.');
   return;
end  

if istruct(n).anchored == 1
   disp ('M3D: select object is anchored, not movable.');
   return;
end
  
if animate
   animatestep = 5;
   theta = t/animatestep;
else
    animatestep = 1;
end

collided  =0;

for kstep = 1:animatestep
   
   if collided == 0
      if istruct(n).linkstart == 0
         rotation_for_structure(n, rotdir, theta, pivpt);
      
      else

         linknumber = istruct(n).linkstart;

         for jj=1:numstruct
            arraysize = size(istruct(jj).link);      
            if linknumber <= arraysize(2)
               if istruct(jj).link(linknumber)
                  rotation_for_structure(jj, rotdir, theta, pivpt);
               end
            end
         end  % for jj loop
      end
   
      collided = detectcollision;
   
      %if collided == 1
      %   disp ('got here');
      %end
   
      drawnow;
   end
end  % for animate loop

return


function rotation_for_structure(i, rotdir, theta, pivpt)

global istruct;
global ibox;
global ireflector;
global raytracing;
global m3debug;

%rotate the bounding box of this structure
rotate (istruct(i).bbox, rotdir, theta, pivpt);

% redraw each box within the rotated structure. 
for j=istruct(i).firstbox:istruct(i).lastbox
   rotate (ibox(j).hdl, rotdir, theta, pivpt);
end
               
% now rotate the vertices of reflectors
if istruct(i).reflecting
   for j=istruct(i).firstreflect:istruct(i).lastreflect
      rotate (ireflector(j).hdl, rotdir, theta, pivpt);
   end
   if raytracing
      raypathupdate;
   end
end
               
% move the rotation orientation of each structure accordingly
if ~isempty(istruct(i).dirhdl)
   rotate (istruct(i).dirhdl, rotdir, theta, pivpt);
end

return               