function m3trans(tarray)
% M3TRANS translate the previously selected structure by 
% the specified amount. 
%
%  format: m3trans (amount-of-translation) 
%  example: m3trans ([10 0 0]) 
%
%  See also MAT3D, M3ROTATE, SEL, DETECTCONTACT

% Copyright (c)1999, Nanping R. Lo nanping@ee.ucla.edu

if nargin < 1, error('m3trans accepts 1 input.'); end

global selectedObj;
global numstruct;
global istruct; 
global animate;

if selectedObj == 0
   disp ('M3D error: no object selected for translation.');
   return;
end  

n = selectedObj;

if istruct(n).anchored == 1
   disp ('M3D: select object is anchored, not movable.');
   return;
end
      
if animate
   animatestep = 5;
   movex = tarray(1)/animatestep;
   movey = tarray(2)/animatestep;
   movez = tarray(3)/animatestep;
   t = [1 0 0 movex; 0 1 0 movey; 0 0 1 movez; 0 0 0 1];
         
else
   animatestep = 1;
   movex = tarray(1);
   movey = tarray(2);
   movez = tarray(3);
   t = [1 0 0 tarray(1); 0 1 0 tarray(2); 0 0 1 tarray(3); 0 0 0 1];
end

collided  =0;

for kstep = 1:animatestep
   if collided == 0   
      if istruct(n).linkstart == 0
          translation_for_structure(n, movex, movey, movez);
      else
         linknumber = istruct(n).linkstart;

         for jj=1:numstruct
            arraysize = size(istruct(jj).link);
            if linknumber <= arraysize(2)
               if istruct(jj).link(linknumber)
                  translation_for_structure(jj, movex, movey, movez);
               end
            end
         end
      end
      collided = detectcollision;
   
      %if collided == 1
      %   disp ('got here');
      %end
      drawnow;
   end
end   % for kstep
return


function translation_for_structure(i, movex, movey, movez)

global istruct; 
global ibox;
global ireflector;
global raytracing;

%translate the bounding box of the structure
newx = get(istruct(i).bbox, 'xdata') + movex;
newy = get(istruct(i).bbox, 'ydata') + movey;
newz = get(istruct(i).bbox, 'zdata') + movez;
set( istruct(i).bbox, 'Xdata', newx, 'Ydata', newy, 'Zdata', newz);

%translate each box within the structure
for j=istruct(i).firstbox:istruct(i).lastbox
   newx = get(ibox(j).hdl, 'xdata') + movex;
   newy = get(ibox(j).hdl, 'ydata') + movey;
   newz = get(ibox(j).hdl, 'zdata') + movez;
   set( ibox(j).hdl, 'Xdata', newx, 'Ydata', newy, 'Zdata', newz);
end
               
% now translate the vertices of reflector
if istruct(i).reflecting
   for l=istruct(i).firstreflect:istruct(i).lastreflect
      newx = get(ireflector(l).hdl, 'xdata') + movex;
      newy = get(ireflector(l).hdl, 'ydata') + movey;
      newz = get(ireflector(l).hdl, 'zdata') + movez;
      set( ireflector(l).hdl, 'Xdata', newx, 'Ydata', newy, 'Zdata', newz);
   end
   if raytracing
      raypathupdate;
   end
end
               
% move the pivot point of each structure accordingly
if ~isempty(istruct(i).dirhdl)
   newx = get(istruct(i).dirhdl, 'xdata') + movex;
   newy = get(istruct(i).dirhdl, 'ydata') + movey;
   newz = get(istruct(i).dirhdl, 'zdata') + movez;
   set( istruct(i).dirhdl, 'Xdata', newx, 'Ydata', newy, 'Zdata', newz);
end

return