function mat3d(filename)
% MAT3D  Matlab toolkit to allow display of CIF file with 
%  topology information as well as allow manipulation, and
%  collision detection of structures within the plot.
%
%  invokation:  mat3d inputfilename
%
%  See also M3ROTATE, M3TRANS, DETECTCONTACT

% Copyright (c)1999, Nanping R. Lo nanping@ee.ucla.edu

if nargin == 0
   clear all;
   infilename = input('Please enter the data file name: ','s');
else
   outfid = fopen ('m3file.dat', 'w');
   fprintf(outfid,'%s', filename);
   fclose (outfid);
   clear all;
   infid = fopen( 'm3file.dat','r');
   infilename=fgetl(infid);
   fclose(infid);
end

global hilitedObj;
global selectedObj;
global numbox;
global numstruct;
global numreflector;
global numray;
global ibox;
global ireflector;
global istruct;
global colors;
global animate;
global raytracing;
global coordmax;
global maxreflect;
global detectcontact;
global createArrowHandle;
global comparison_cnt;
global mat3dversion;
global mat3ddate;

mat3dversion = '0.11';
mat3ddate = 'September 14, 1999';
selectedObj = 0;
hilitedObj =0;
numbox=0;
numreflector=0;
numray = 0;
numstruct=0;
coordmax = 50;
numlink = 0;
animate = 0;
raytracing = 0;
maxreflect = 100;
detectcontact = 0;
createArrowHandle = 1;

%define colors
colors.yellow = [0.9 0.9 0.1];
colors.red    = [1 0.4 0.2];
colors.blue   = [0.6 0.8 1];
colors.green  = [0.2 0.9 0.2];
colors.brown  = [0.9 0.9 0.7];

% facet
%lfs=[1 2 6 5;2 3 7 6;3 4 8 7;4 1 5 8;1 2 3 4;5 6 7 8];
%one=[1 1 1 1 1 1 1 1];

lfs = [1 2 3 4; 5 6 7 8; 9 10 11 12; 13 14 15 16; 17 18 19 20; 21 22 23 24];
one = [1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1];

%initialize graphics
figure(99)
clf

% read the data file
fid = fopen(infilename,'r');

while feof(fid) == 0
  line=fgetl(fid);
  tmpval = sscanf(line, '%s');   
  linelength = length(line);
  
  if (linelength == 0)
      % ignor anything blank lines
  elseif strcmp (tmpval(1), '#')
      % ignor anything started with '#' 
  elseif strcmp (tmpval(1), '%')
      % ignor anything started with '%' as well
  else

    if linelength == 1
      if strcmp(tmpval,'{')
         read_reflect_pieces = 0;
         numstruct = numstruct+1;
         % struct(1) contains the ordinate number of the 1st
         % box in the structure
         istruct(numstruct).firstbox = numbox+1;
         istruct(numstruct).reflecting = 0;
         istruct(numstruct).linkstart  = 0;
          
      elseif strcmp(tmpval, '}')
         % struct(2) contains the ordinate number of the last
         % box in the structure
         istruct(numstruct).lastbox = numbox;
      end
      
    elseif findstr('anchor:', line)
      if findstr( 'false', line)
         % not anchore
         istruct(numstruct).anchored = 0;
      elseif findstr('true', line)
         % is anchore
         istruct(numstruct).anchored = 1;
      end
      
    elseif findstr('link', line)
      tmpstr = sscanf(line, '%s %d %s'); 
      linknumber = tmpstr(5);
      
      if findstr ('start', line)
         istruct(numstruct).linkstart = linknumber;
         clear istruct(numstruct).link(linknumber);
         istruct(numstruct).link(linknumber) = 1;  % a row matrix
      elseif findstr ('member', line)
         istruct(numstruct).link(linknumber) = 1;  % a row matrix
      end
      
   elseif findstr ('reflect start', line)
      read_reflect_pieces = 1;
      istruct(numstruct).reflecting = 1;
      istruct(numstruct).firstreflect = numreflector+1;
      
   elseif findstr ('reflect end', line)
      read_reflect_pieces = 0;
      istruct(numstruct).lastreflect = numreflector;
      
   elseif findstr ('rotate:', line)
      if findstr     ('+x', line)
         istruct (numstruct).rotatedir = '+x';
         rdir = [1 0 0];
      elseif findstr ('-x', line)
         istruct (numstruct).rotatedir = '-x';   
         rdir = [1 0 0];
      elseif findstr ('+y', line)
         istruct (numstruct).rotatedir = '+y';
         rdir = [0 1 0];
      elseif findstr ('-y', line)
         istruct (numstruct).rotatedir = '-y';
         rdir = [0 1 0];
      elseif findstr ('+z', line)
         istruct (numstruct).rotatedir = '+z';
         rdir = [0 0 1];
      else
         istruct (numstruct).rotatedir = '-z';
         rdir = [0 0 1];
      end
      %line
      tmpstr = sscanf(line, '%s%s%g%g%g');
      %istruct(numstruct).pivot(1) = tmpstr(10);
      %istruct(numstruct).pivot(2) = tmpstr(11);
      %istruct(numstruct).pivot(3) = tmpstr(12);
      
      xpt = [tmpstr(10) tmpstr(10)+rdir(1)];
      ypt = [tmpstr(11) tmpstr(11)+rdir(2)];
      zpt = [tmpstr(12) tmpstr(12)+rdir(3)];
      istruct(numstruct).dirhdl = patch ('XData',xpt,'YData',ypt,'ZData',zpt,'visible', 'off');

   elseif linelength > 1
      tmpval = sscanf(line,'%g', [1 6]);
      if read_reflect_pieces
        numreflector = numreflector +1;
        xmax = tmpval(4) + tmpval(1)/2.0;
        xmin = tmpval(4) - tmpval(1)/2.0;
        ymax = tmpval(5) + tmpval(2)/2.0;
        ymin = tmpval(5) - tmpval(2)/2.0;
        zval = tmpval(6) + tmpval(3)/2.0;
        vert  = [xmax ymax zval; xmin ymax zval; xmin ymin zval; xmax ymin zval];
        ireflector(numreflector).hdl = patch ('faces', [1 2 3 4], 'vertices', vert, ...
                                              'FaceColor', 'y', 'visible', 'off');
      else
        numbox = numbox+1;
        ll(numbox) = tmpval(1);
        ww(numbox) = tmpval(2);
        hh(numbox) = tmpval(3);
        xx(numbox) = tmpval(4);
        yy(numbox) = tmpval(5);
        zz(numbox) = tmpval(6);
    
        coordmax = max(abs(tmpval(4)),coordmax);
        coordmax = max(abs(tmpval(5)),coordmax);      
      end
    end
  end
end

fclose(fid);

for n = 1:numbox

   % color the boxes according to the height!
   switch zz(n)
      case 1      
         boxcolor = colors.yellow;  % layer c1
      case 2
         boxcolor = colors.red; 
      case 2.5
         boxcolor = colors.yellow;
      case 4
         if hh(n) == 6.0
            boxcolor = colors.yellow;  % layer c21
         else
            boxcolor = colors.blue;
         end
      case -8
         boxcolor = colors.brown;
      otherwise
         boxcolor = colors.blue;      
   end
   
   %xdata=[0 ll(n) ll(n) 0     0     ll(n) ll(n) 0];
   %ydata=[0 0     ww(n) ww(n) 0     0     ww(n) ww(n)];
   %zdata=[0 0     0     0     hh(n) hh(n) hh(n) hh(n)];
   %linkdata=[xdata;ydata;zdata;one];
   
   x2 = ll(n);
   y2 = ww(n);
   z2 = hh(n);
   x1 = 0;
   y1 = 0;
   z1 = 0;
   xdata = [x1 x2 x2 x1 x2 x2 x2 x2 x2 x1 x1 x2 x1 x1 x1 x1 x1 x2 x2 x1 x1 x2 x2 x1];
   ydata = [y1 y1 y1 y1 y1 y2 y2 y1 y2 y2 y2 y2 y2 y1 y1 y2 y1 y1 y2 y2 y1 y1 y2 y2];
   zdata = [z1 z1 z2 z2 z1 z1 z2 z2 z1 z1 z2 z2 z1 z1 z2 z2 z1 z1 z1 z1 z2 z2 z2 z2];

   %xdata = [0 x x 0 x x x x x 0 0 x 0 0 0 0 0 x x 0 0 x x 0];
   %ydata = [0 0 0 0 0 y y 0 y y y y y 0 0 y 0 0 y y 0 0 y y];
   %zdata = [0 0 z z 0 0 z z 0 0 z z 0 0 z z 0 0 0 0 z z z z];
   linkdata = [xdata; ydata; zdata; one];

   %Translate link
   tmpx=xx(n)-(ll(n)/2);
   tmpy=yy(n)-(ww(n)/2);
   tmpz=zz(n)-(hh(n)/2);
   t=[1 0 0 tmpx; 0 1 0 tmpy; 0 0 1 tmpz; 0 0 0 1];
   %end translate
   
   clear vermat;
   clear ldata;
   
   ldata=t*linkdata;
   vermat = [ldata(1,:)' ldata(2,:)' ldata(3,:)'];
   
   % Setting up handles for animation
   ibox(n).hdl = patch('faces', lfs, 'vertices', vermat, 'FaceColor', boxcolor, 'visible', 'on' );
end

for k = 1:numstruct
   firstflag = 1;
   for m=istruct(k).firstbox:istruct(k).lastbox
      maxpt = [ xx(m)+ll(m)/2 yy(m)+ww(m)/2 zz(m)+hh(m)/2];
      minpt = [ xx(m)-ll(m)/2 yy(m)-ww(m)/2 zz(m)-hh(m)/2];
      if firstflag
         bboxmax = maxpt;
         bboxmin = minpt;
         firstflag = 0;
      end
      bboxmax = max (bboxmax, maxpt);
      bboxmin = min (bboxmin, minpt);
   end
   
   x1 = bboxmin(1);
   x2 = bboxmax(1);
   y1 = bboxmin(2);
   y2 = bboxmax(2);
   z1 = bboxmin(3);
   z2 = bboxmax(3);
   xdata = [x1 x2 x2 x1 x2 x2 x2 x2 x2 x1 x1 x2 x1 x1 x1 x1 x1 x2 x2 x1 x1 x2 x2 x1];
   ydata = [y1 y1 y1 y1 y1 y2 y2 y1 y2 y2 y2 y2 y2 y1 y1 y2 y1 y1 y2 y2 y1 y1 y2 y2];
   zdata = [z1 z1 z2 z2 z1 z1 z2 z2 z1 z1 z2 z2 z1 z1 z2 z2 z1 z1 z1 z1 z2 z2 z2 z2];
   vermat = [xdata; ydata; zdata]';
   istruct(k).bbox = patch('faces', lfs, 'vertices', vermat, 'visible','off');
end

clear ll;
clear ww; 
clear hh; 
clear xx;
clear yy;
clear zz;

tmpnumb = 50;

% a kludge, if I know how to to modulas & remainder in matlab,
% this would be done differently
while (coordmax > tmpnumb)
   tmpnumb = tmpnumb+25;
end

coordmax =tmpnumb;

hold on
set(gcf,'renderer','zbuffer');
grid on
%view(30, 60)
view(0, 90)
title('3D micro structure viewing with matlab');
xlabel('X-axis')
ylabel('Y-axis')
zlabel('Z-axis')
axis([-coordmax coordmax -coordmax coordmax 0 coordmax]);

if (isunix ~= 1)
  daspect([1 1 1])
end 

rotate3d on

%tmpstr = sprintf ('there are %d structures and %d boxes in the scene.', numstruct, numbox);
%disp (tmpstr);
%writeCFfile('cftmp.dat');

%cnt=0;
%for n = 1:numstruct
%   tmp = istruct(n);       
%   for j=tmp.firstbox:tmp.lastbox
%      cnt = cnt+1;
%      v = ibox(j).vermat;
%      for i = 1:3
%         for k = 1:8
%            vv(cnt,(i-1)*8+k) = v(k, i);
%         end
%      end
%   end
%end
%
%cf_init = 1;
%cf_verbose = 1;
%detect( cf_init, cf_verbose, numbox, [1], vv');
%pause
%detect(0, cf_verbose, 0, [0.0 0.0 0.0 0.0 0.0 0.0], [1]);
