function [hit, coord] = raybox (minB, maxB, origin, dir)
% RAYBOX is an internal rountine of mat3d for raytracing.
%
%  See also MAT3D, RAYSOURCE, RAYTRACING

% Copyright (c)1999, Nanping R. Lo nanping@ee.ucla.edu


hit = [];
coord = origin;

if nargin ~= 4, error('raybox accepts 4 input.'); end
   
rb_numdim = 3;
rb_right = 0;
rb_left = 1;
rb_middle =2;

inside = 1; 
quadrant(1) =0;
quadrant(2) =0;
quadrant(3) =0;

whichPlane = 0;
maxT(1) = 0;
maxT(2) = 0;
maxT(3) = 0;
candidatePlane(1) =0;
candidatePlane(2) =0 ;
candidatePlane(3) =0;

%% Find candidate planes; this loop can be avoided if
%% rays cast all from the eye(assume perpsective view)

for i=1:rb_numdim
   if(origin(i) < minB(i))
      quadrant(i) = rb_left;
      candidatePlane(i) = minB(i);
      inside = 0;
   elseif (origin(i) > maxB(i)) 
      quadrant(i) = rb_right;
      candidatePlane(i) = maxB(i);
      inside = 0;
   else
      quadrant(i) = rb_middle;      
   end
   
end

%% Ray origin inside bounding box 
if(inside)
   coord = origin;
   hit = 0;
   return
end


%% Calculate T distances to candidate planes 
for i = 1:rb_numdim
  if (quadrant(i) ~= rb_middle & dir(i) ~=0.)
      maxT(i) = (candidatePlane(i)-origin(i)) / dir(i);
  else
      maxT(i) = -1.;      
  end   
end

%% Get largest of the maxT's for final choice of intersection

whichPlane = 1;
for i = 1:rb_numdim
   if (maxT(whichPlane) < maxT(i))
      whichPlane = i;
   end
end

%% Check final candidate actually inside box */
if (maxT(whichPlane) < 0.) 
   hit =0;
   return
end

for i = 1:rb_numdim
   if (whichPlane ~= i) 
      coord(i) = origin(i) + maxT(whichPlane) *dir(i);
      if (coord(i) < minB(i) | coord(i) > maxB(i))
         hit = 0;
         return
      end
   else
      coord(i) = candidatePlane(i);
   end

end

hit = 1;

return