function raypathupdate()
% RAYPATHUPDATE is an internal rountine of mat3d for raytracing.
%
%  See also MAT3D, RAYSOURCE, RAYTRACING

% Copyright (c)1999, Nanping R. Lo nanping@ee.ucla.edu

global ray;
global numray;
global numreflector;
global coordmax;
global ireflector;
global maxreflect;

if (nargin ~= 0)
   disp('M3D error: incorrect number of inputs.');
   return;
end

%disp ('M3D: raytracing not yet implemented.');

if numray == 0
   disp('M3D: ray source not defined.');
else
   for k =1:numray
      clear lx;
      clear ly;
      clear lz;
      
      lx(1) = ray(k).start(1);
      ly(1) = ray(k).start(2);
      lz(1) = ray(k).start(3);
      lx(2) = lx(1) + ray(k).dir(1)* 4*coordmax;
      ly(2) = ly(1) + ray(k).dir(2)* 4*coordmax;
      lz(2) = lz(1) + ray(k).dir(3)* 4*coordmax;
      
      if ray(k).trace
         j=2;
         rs =ray(k).start;
         rd =ray(k).dir;
         m = 1;
         current_reflector = 0;

         while m <= numreflector
            hit = 0;
            
            if (m ~= current_reflector)
               clear vertmat;
               vertmat = get(ireflector(m).hdl, 'vertices');
               pa = vertmat(1,:);
               pb = vertmat(2,:);
               pc = vertmat(3,:);
               pd = vertmat(4,:);
               [hit,intersect,newdir] = rayfacet(pa,pb,pc,pd,rs,rd);
            end
            
            if (hit)
               current_reflector = m;
               lx(j)   = intersect(1); 
               ly(j)   = intersect(2);
               lz(j)   = intersect(3);
               lx(j+1) = lx(j) + newdir(1) *4*coordmax;
               ly(j+1) = ly(j) + newdir(2) *4*coordmax;
               lz(j+1) = lz(j) + newdir(3) *4*coordmax;
               rs=[intersect(1) intersect(2) intersect(3)];
               rd=[newdir(1) newdir(2) newdir(3)];
               j=j+1;
               m=1;
            else
               m=m+1;
            end
            
            if j > maxreflect
               disp ('M3D: existing ray reflecting, maximum ''maxreflect'' reached.');
               break
            end
            
         end
         
      else
         lx(2) = lx(1);
         ly(2) = ly(1);
         lz(2) = lz(1);   
      end
      
      set (ray(k).linehdl, 'XData', lx, 'YData', ly, 'ZData',lz);
      
   end
   
end