function raysource(rayidx, start, dir)
% RAYSOURCE - specifies the location and the direction of the ray.
%
%  format: raysource (ray-source-num, ray-origin-point, ray-dir-vector)
%          for now, ray-source-number is always 1.  
%  example: raysource (1, [200 100 100], [1 1 -1]) 
%
%  See also MAT3D, RAYTRACING

% Copyright (c)1999, Nanping R. Lo nanping@ee.ucla.edu


global coordmax;
global ray;
global raytracing;
global numray;

if (nargin ~= 3)
   disp('M3D error: incorrect number of inputs.');
   return;
end

%original orientation of the cone
originaldir =[0 0 1];

epsilon = 0.0001;
if norm(dir) < epsilon
   disp('M3D error: norm of direction vector cannot be zero.');
   return;
else
   newdir=dir/norm(dir);
end



R = [10 10 10 12 12 1 1];
lengthR= length(R);

N =10;
scaleX = coordmax/250 * 0.5;
scaleY = coordmax/250 * 0.5;
scaleZ = coordmax/250 * 24.0;

offsetX=start(1);
offsetY=start(2);
offsetZ=start(3);

[x y z] = cylinder(R, N);
c = ones(lengthR,N+1);

x = scaleX*x + offsetX;
y = scaleY*y + offsetY;
z = scaleZ*z + offsetZ;

if ~isstruct(ray) | rayidx > length(ray) | isempty(ray(rayidx).srchdl)
   ray(rayidx).srchdl  = surf(x, y, z, c);
   numray = numray +1;
else
   set (ray(rayidx).srchdl,  'XData', x, 'YData', y, 'ZData',z);
end

[theta, phi, rad] = cart2sph(newdir(1), newdir(2), newdir(3));

theta = theta * 180/pi - 90.0;
phi   = phi   * 180/pi - 90.0;

rotate (ray(rayidx).srchdl, [1 0 0], phi,   start);
rotate (ray(rayidx).srchdl, [0 0 1], theta, start);

% now initialize the ray trace
lx = [start(1) start(1)];
ly = [start(2) start(2)];
lz = [start(3) start(3)];

ray(rayidx).start   = start;
ray(rayidx).dir     = dir;
ray(rayidx).trace   = raytracing;
ray(rayidx).linehdl = line(lx, ly, lz, 'Color', 'r', 'LineWidth', 1);

raypathupdate;
