function setlink ( which, linkstr)
% SETLINKAGE sets the linkage information for the structures being plotted.
%  The specified linkage information will overwrite the linkage information
%  read from the input file
%
%  format: setlink [start m|memberof n|notmemberof k]
%
%  example: setlink start 2
%           setlink memberof 3
%           setlink notmemberof 4
%
%  See also MAT3D, SEL, SHOWLINKAGE

% Copyright (c)1999, Nanping R. Lo nanping@ee.ucla.edu

global numstruct;
global istruct;
global m3debug;
global selectedObj;

if (nargin ~= 1 & nargin ~= 2)
   disp('M3D: incorrect number of inputs.');
   return;
end

linknumber = str2num (linkstr);

if (isempty (linknumber))
   disp('M3D: the 2nd parameter of linkset needs to be numeric.');
   return;
end

if selectedObj == 0 
   disp ('M3d: please select a structure first.'); 
   return;
end

n = selectedObj;

if strcmp(which, 'start')
   istruct(n).linkstart = linknumber;
   clear istruct(n).link(linknumber);
   istruct(n).link(linknumber) = 1;  % a row matrix

elseif strcmp (which, 'memberof')
   if istruct(n).linkstart == 0
      disp ('M3D: please set ''member start'' for this structure first.');
      return;
   end
   clear istruct(n).link(linknumber); % wipe out the old membership
   istruct(n).link(linknumber) = 1;
   
elseif strcmp (which, 'notmemberof')
   if istruct(n).linkstart == 0
      disp ('M3D: please set ''member start'' for this structure first.');
      return;
   end
   istruct(n).link(linknumber) = 0;
   
else
   disp ('M3D: please specify either ''setlink start'' or ''setlink memberof n''.');
end

return;
