function setpivot( a, p)
% SETPIVOT sets the rotation direction as well as the pivot
%  point of the previously selected structure.  It over-write
%  in memory the "structure rotate" read from the input file.
%
%  format: setpivot (axis, pivot-point) 
%  example: setpivot ('x', [10 10 2]) 
%
%  See also MAT3D, SEL, SHOWPIVOT, M3ROTATE, M3TRANS

% Copyright (c)1999, Nanping R. Lo nanping@ee.ucla.edu

global numstruct;
global istruct;
global m3debug;
global selectedObj;

if (nargin ~= 2)
   error('M3D: incorrect number of inputs.');
end

n = selectedObj;

if n > numstruct
   disp ('M3D: no such structure (the syntax is ''sel(n)'', with parenthsis).');
   return;
end

if istruct(n).anchored == 1
   disp ('M3D: select object is anchored, not movable. pivot not set');
   return;
end

if findstr ('x', a)
   istruct (n).rotatedir = '+x';
   rdir = [1 0 0];
elseif findstr ('y', a)
   istruct (n).rotatedir = '+y';
   rdir = [0 1 0];
elseif findstr ('z', a)
   istruct (n).rotatedir = '+z';
   rdir = [0 0 1];
end

xpt = [p(1) p(1)+rdir(1)];
ypt = [p(2) p(2)+rdir(2)];
zpt = [p(3) p(3)+rdir(3)];
istruct(n).dirhdl = patch ('XData',xpt,'YData',ypt,'ZData',zpt,'visible', 'off');

return;
