function setreflector( box )
% SETREFLECTOR sets the reflector box geometry of the currently selected
%  structure.
%  If previously set, this command over-writes the reflector geometry information
%  in memory read from the input file.
%
%  format: setreflector ( box ) 
%  where box is specified as [width lenght width xcenter ycenter zcenter]
%
%  example: setreflector ( [10 10 1 -50 20 2] ) 
%
%  See also MAT3D, SEL, RAYTRACYING, RAYSOURCE

% Copyright (c)1999, Nanping R. Lo nanping@ee.ucla.edu

global istruct;
global selectedObj;
global numreflector;
global ireflector;

if (nargin ~= 1)
   error('M3D: incorrect number of inputs.');
end

n = selectedObj;

xmax = box(4) + box(1)/2.0;
xmin = box(4) - box(1)/2.0;
ymax = box(5) + box(2)/2.0;
ymin = box(5) - box(2)/2.0;
zval = box(6) + box(3)/2.0;

vert  = [xmax ymax zval; xmin ymax zval; xmin ymin zval; xmax ymin zval];

if (istruct(n).reflecting == 1 )
   k = istruct(n).firstreflect;
   set (ireflector(k).hdl,  'vertices', vert, 'visible', 'off' );
else
   numreflector = numreflector +1;
   istruct(numstruct).firstreflect = numreflector;
   istruct(numstruct).lastreflect  = numreflector;
   ireflector(k).hdl = patch ('faces', [1 2 3 4], 'vertices', vert, ...
                              'FaceColor', 'y', 'visible', 'off');
end

return;
