function showlinkage( state )
% SHOWLINKAGE prints out the linkage information, with no parameter specified,
%  SHOWLINKAGE will return the linkage information for the last structure
%  selected.  If 'all' is specified, linkage information for all strucures
%  are returned.
%
%  format: showlinkage [all]
%
%  example: showlinkage all
%           showlinkage
%
%  See also MAT3D, SEL, SETLINK 

% Copyright (c)1999, Nanping R. Lo nanping@ee.ucla.edu

global numstruct;
global m3debug;
global selectedObj;

if (nargin > 1)
   error('M3D: incorrect number of inputs.');
end

if (nargin == 0)
   if (selectedObj == 0)
      disp ('M3D: no object currently selected.');
      return;
   end
   
   linkage_for_structure ( selectedObj);

elseif strcmp(state, 'all')

   for k=1:numstruct
       linkage_for_structure (k);
   end
else
   disp ('M3D: the second parameter of showlinkage needs to be ''all''.');
end


return;


function linkage_for_structure(i)

global istruct;
global numstruct;
global m3debug;


linknumber = istruct(i).linkstart;

if linknumber == 0 
   tstr = sprintf ('struct %d: not part of a defined linkage', i);
   disp (tstr); 
else
   
   tstr =sprintf ('');
   for j=1:numstruct
      arraysize = size(istruct(j).link);      
      if linknumber <= arraysize(2)
         if istruct(j).link(linknumber)
            tstr = sprintf ('%s %d', tstr, j);
         end
      end
   end 
   tstr = sprintf ('struct %d: part of link %d which consist of struct(s):%s', i, linknumber, tstr);
   disp (tstr);
end
return;